@@ -83,6 +83,12 @@ Block::Block(const UUID &uu, const json &j, Pool &pool) : uuid(uu), name(j.at("n
             Logger::log_warning("couldn't load bom export settings", Logger::Domain::BLOCK, e.what());
         }
     }
+    if (j.count("project_meta")) {
+        const json &o = j["project_meta"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            project_meta[it.key()] = it.value();
+        }
+    }
 }
 
 void Block::update_diffpairs()
@@ -111,7 +117,7 @@ Block::Block(const UUID &uu) : uuid(uu)
 Block Block::new_from_file(const std::string &filename, Pool &obj)
 {
     auto j = load_json_from_file(filename);
-    return Block(UUID(j["uuid"].get<std::string>()), j, obj);
+    return Block(UUID(j.at("uuid").get<std::string>()), j, obj);
 }
 
 Net *Block::get_net(const UUID &uu)
@@ -216,7 +222,7 @@ void Block::update_connection_count()
 Block::Block(const Block &block)
     : uuid(block.uuid), name(block.name), nets(block.nets), buses(block.buses), components(block.components),
       net_classes(block.net_classes), net_class_default(block.net_class_default), group_names(block.group_names),
-      tag_names(block.tag_names), bom_export_settings(block.bom_export_settings)
+      tag_names(block.tag_names), project_meta(block.project_meta), bom_export_settings(block.bom_export_settings)
 {
     update_refs();
 }
@@ -232,6 +238,7 @@ void Block::operator=(const Block &block)
     net_class_default = block.net_class_default;
     group_names = block.group_names;
     tag_names = block.tag_names;
+    project_meta = block.project_meta;
     bom_export_settings = block.bom_export_settings;
     update_refs();
 }
@@ -267,6 +274,7 @@ json Block::serialize()
         j["tag_names"][(std::string)it.first] = it.second;
     }
     j["bom_export_settings"] = bom_export_settings.serialize();
+    j["project_meta"] = project_meta;
 
     return j;
 }
@@ -295,6 +303,9 @@ std::map<const Part *, BOMRow> Block::get_BOM(const BOMExportSettings &settings)
 {
     std::map<const Part *, BOMRow> rows;
     for (const auto &it : components) {
+        if (it.second.nopopulate && !settings.include_nopopulate) {
+            continue;
+        }
         if (it.second.part) {
             const Part *part;
             if (settings.concrete_parts.count(it.second.part->uuid))
@@ -342,4 +353,41 @@ std::string Block::get_tag_name(const UUID &uu) const
         return (std::string)uu;
 }
 
+std::map<std::string, std::string> Block::peek_project_meta(const std::string &filename)
+{
+    auto j = load_json_from_file(filename);
+    if (j.count("project_meta")) {
+        const json &o = j["project_meta"];
+        std::map<std::string, std::string> project_meta;
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            project_meta[it.key()] = it.value();
+        }
+        return project_meta;
+    }
+    return {};
+}
+
+std::string Block::get_net_name(const UUID &uu) const
+{
+    auto &net = nets.at(uu);
+    if (net.name.size()) {
+        return net.name;
+    }
+    else {
+        std::string n;
+        for (const auto &it : components) {
+            for (const auto &it_conn : it.second.connections) {
+                if (it_conn.second.net && it_conn.second.net->uuid == uu) {
+                    n += it.second.refdes + ", ";
+                }
+            }
+        }
+        if (n.size()) {
+            n.pop_back();
+            n.pop_back();
+        }
+        return n;
+    }
+}
+
 } // namespace horizon
