@@ -28,6 +28,7 @@ public:
     Block(const UUID &uu, const json &, Pool &pool);
     Block(const UUID &uu);
     static Block new_from_file(const std::string &filename, Pool &pool);
+    static std::map<std::string, std::string> peek_project_meta(const std::string &filename);
     Net *get_net(const UUID &uu);
     UUID uuid;
     std::string name;
@@ -39,6 +40,7 @@ public:
 
     std::map<UUID, std::string> group_names;
     std::map<UUID, std::string> tag_names;
+    std::map<std::string, std::string> project_meta;
     std::string get_group_name(const UUID &uu) const;
     std::string get_tag_name(const UUID &uu) const;
 
@@ -73,6 +75,8 @@ public:
      */
     Net *insert_net();
 
+    std::string get_net_name(const UUID &uu) const;
+
     json serialize();
 
 private:
