@@ -25,7 +25,7 @@ json Board::StackupLayer::serialize() const
     return j;
 }
 
-Board::Colors::Colors() : solder_mask({0, .5, 0}), substrate({.2, .15, 0})
+BoardColors::BoardColors() : solder_mask({0, .5, 0}), substrate({.2, .15, 0})
 {
 }
 
@@ -57,6 +57,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
                          Logger::Domain::BOARD);
         }
     }
+    std::set<UUID> texts_skip;
     if (j.count("packages")) {
         const json &o = j["packages"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
@@ -67,6 +68,8 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
                              Logger::Domain::BOARD);
             }
             else {
+                auto texts_lost = BoardPackage::peek_texts(it.value());
+                std::copy(texts_lost.begin(), texts_lost.end(), std::inserter(texts_skip, texts_skip.end()));
                 Logger::log_warning("not loading package " + (std::string)u + " since component is gone");
             }
         }
@@ -120,7 +123,8 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
         const json &o = j["texts"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            load_and_log(texts, ObjectType::TEXT, std::forward_as_tuple(u, it.value()), Logger::Domain::BOARD);
+            if (!texts_skip.count(u))
+                load_and_log(texts, ObjectType::TEXT, std::forward_as_tuple(u, it.value()), Logger::Domain::BOARD);
         }
     }
     if (j.count("lines")) {
@@ -168,6 +172,22 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
                          Logger::Domain::BOARD);
         }
     }
+    if (j.count("included_boards")) {
+        const json &o = j["included_boards"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            load_and_log(included_boards, ObjectType::BOARD, std::forward_as_tuple(u, it.value()),
+                         Logger::Domain::BOARD);
+        }
+    }
+    if (j.count("board_panels")) {
+        const json &o = j["board_panels"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            load_and_log(board_panels, ObjectType::BOARD, std::forward_as_tuple(u, it.value(), *this),
+                         Logger::Domain::BOARD);
+        }
+    }
     if (j.count("rules")) {
         try {
             rules.load_from_json(j.at("rules"));
@@ -211,6 +231,14 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
             Logger::log_warning("couldn't load step export settings", Logger::Domain::BOARD, e.what());
         }
     }
+    if (j.count("pnp_export_settings")) {
+        try {
+            pnp_export_settings = PnPExportSettings(j.at("pnp_export_settings"));
+        }
+        catch (const std::exception &e) {
+            Logger::log_warning("couldn't load p&p export settings", Logger::Domain::BOARD, e.what());
+        }
+    }
     fab_output_settings.update_for_board(this);
     update_pdf_export_settings(pdf_export_settings);
     update_refs(); // fill in smashed texts
@@ -219,7 +247,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
 Board Board::new_from_file(const std::string &filename, Block &block, Pool &pool, ViaPadstackProvider &vpp)
 {
     auto j = load_json_from_file(filename);
-    return Board(UUID(j["uuid"].get<std::string>()), j, block, pool, vpp);
+    return Board(UUID(j.at("uuid").get<std::string>()), j, block, pool, vpp);
 }
 
 Board::Board(const UUID &uu, Block &bl) : uuid(uu), block(&bl)
@@ -249,18 +277,39 @@ const std::map<int, Layer> &Board::get_layers() const
     return layers;
 }
 
-Board::Board(const Board &brd)
+Board::Board(const Board &brd, CopyMode copy_mode)
     : layers(brd.layers), uuid(brd.uuid), block(brd.block), name(brd.name), polygons(brd.polygons), holes(brd.holes),
-      packages(brd.packages), junctions(brd.junctions), tracks(brd.tracks), airwires(brd.airwires), vias(brd.vias),
-      texts(brd.texts), lines(brd.lines), arcs(brd.arcs), planes(brd.planes), keepouts(brd.keepouts),
-      dimensions(brd.dimensions), connection_lines(brd.connection_lines), warnings(brd.warnings), rules(brd.rules),
-      fab_output_settings(brd.fab_output_settings), stackup(brd.stackup), colors(brd.colors),
+      junctions(brd.junctions), tracks(brd.tracks), texts(brd.texts), lines(brd.lines), arcs(brd.arcs),
+      planes(brd.planes), keepouts(brd.keepouts), dimensions(brd.dimensions), connection_lines(brd.connection_lines),
+      included_boards(brd.included_boards), board_panels(brd.board_panels), warnings(brd.warnings), rules(brd.rules),
+      fab_output_settings(brd.fab_output_settings), airwires(brd.airwires), stackup(brd.stackup), colors(brd.colors),
       pdf_export_settings(brd.pdf_export_settings), step_export_settings(brd.step_export_settings),
-      n_inner_layers(brd.n_inner_layers)
+      pnp_export_settings(brd.pnp_export_settings), n_inner_layers(brd.n_inner_layers)
 {
+    if (copy_mode == CopyMode::DEEP) {
+        packages = brd.packages;
+        vias = brd.vias;
+    }
+    else {
+        for (const auto &it : brd.packages) {
+            packages.emplace(std::piecewise_construct, std::forward_as_tuple(it.first),
+                             std::forward_as_tuple(shallow_copy, it.second));
+        }
+        for (const auto &it : brd.vias) {
+            vias.emplace(std::piecewise_construct, std::forward_as_tuple(it.first),
+                         std::forward_as_tuple(shallow_copy, it.second));
+        }
+    }
     update_refs();
 }
 
+Board::Board(const Board &brd) : Board(brd, CopyMode::DEEP)
+{
+}
+Board::Board(shallow_copy_t sh, const Board &brd) : Board(brd, CopyMode::SHALLOW)
+{
+}
+
 void Board::update_refs()
 {
     for (auto &it : packages) {
@@ -276,7 +325,9 @@ void Board::update_refs()
         it.second.update_refs(*this);
     }
     for (auto &it : airwires) {
-        it.second.update_refs(*this);
+        for (auto &it2 : it.second) {
+            it2.update_refs(*this);
+        }
     }
     for (auto &it : vias) {
         it.second.junction.update(junctions);
@@ -311,6 +362,9 @@ void Board::update_refs()
     for (auto &it : connection_lines) {
         it.second.update_refs(*this);
     }
+    for (auto &it : board_panels) {
+        it.second.included_board.update(included_boards);
+    }
 }
 
 unsigned int Board::get_n_inner_layers() const
@@ -323,6 +377,7 @@ void Board::set_n_inner_layers(unsigned int n)
     n_inner_layers = n;
     layers.clear();
     layers = {{200, {200, "Top Notes"}},
+              {BoardLayers::OUTLINE_NOTES, {BoardLayers::OUTLINE_NOTES, "Outline Notes"}},
               {100, {100, "Outline"}},
               {60, {60, "Top Courtyard"}},
               {50, {50, "Top Assembly"}},
@@ -368,6 +423,7 @@ void Board::update_pdf_export_settings(PDFExportSettings &settings)
                 std::forward_as_tuple(l, Color(0, 0, 0), PDFExportSettings::Layer::Mode::OUTLINE, enable));
     };
 
+    add_layer(BoardLayers::OUTLINE_NOTES);
     add_layer(BoardLayers::L_OUTLINE);
     add_layer(BoardLayers::TOP_PASTE, false);
     add_layer(BoardLayers::TOP_SILKSCREEN, false);
@@ -391,7 +447,9 @@ void Board::update_pdf_export_settings(PDFExportSettings &settings)
 
 void Board::flip_package_layer(int &layer) const
 {
-    if (layer < 0 && layer > -100) {
+    if (layer == BoardLayers::L_OUTLINE)
+        return;
+    else if (layer < 0 && layer > -100) {
         if (n_inner_layers == 0)
             return;
         if (-layer > (int)n_inner_layers)
@@ -484,11 +542,29 @@ void Board::vacuum_junctions()
     }
 }
 
+
+static std::string replace_text_map(const std::string &s, const std::map<std::string, std::string> &m)
+{
+    auto fn = [m](const auto &k) -> std::string {
+        if (m.count(k))
+            return m.at(k);
+        else
+            return "";
+    };
+    return replace_placeholders(s, fn, true);
+}
+
 void Board::expand(bool careful)
 {
     delete_dependants();
     warnings.clear();
 
+    for (auto &it : texts) {
+        it.second.overridden = false;
+    }
+
+    expand_packages();
+
     for (auto &it : junctions) {
         it.second.temp = false;
         it.second.layer = 10000;
@@ -572,8 +648,6 @@ void Board::expand(bool careful)
     vacuum_junctions();
     delete_dependants(); // deletes connection lines
 
-    expand_packages();
-
     for (auto &it : polygons) {
         it.second.usage = nullptr;
     }
@@ -597,6 +671,15 @@ void Board::expand(bool careful)
         }
     }
 
+    for (auto &it : texts) {
+        if (it.second.overridden == false) {
+            if (std::count(it.second.text.begin(), it.second.text.end(), '$')) {
+                it.second.overridden = true;
+                it.second.text_override = replace_text_map(it.second.text, block->project_meta);
+            }
+        }
+    }
+
     if (expand_flags & EXPAND_AIRWIRES)
         update_airwires();
 
@@ -787,8 +870,56 @@ void Board::smash_package(BoardPackage *pkg)
     }
 }
 
+void Board::copy_package_silkscreen_texts(BoardPackage *dest, const BoardPackage *src)
+{
+    // Copy smash status
+    unsmash_package(dest);
+    if (!src->smashed) {
+        return;
+    }
+    dest->smashed = true;
+
+    // Copy all texts from src
+    for (const auto &it : src->texts) {
+        if (!BoardLayers::is_silkscreen(it->layer)) {
+            // Not on top or bottom silkscreen
+            continue;
+        }
+
+        auto uu = UUID::random();
+        auto &x = texts.emplace(uu, uu).first->second;
+        x.from_smash = true;
+        x.overridden = true;
+
+        x.placement = dest->placement;
+        Placement rp = it->placement;
+        Placement ref_placement = src->placement;
+        if (ref_placement.mirror) {
+            ref_placement.invert_angle();
+        }
+        rp.make_relative(ref_placement);
+        if (x.placement.mirror) {
+            x.placement.invert_angle();
+        }
+        x.placement.accumulate(rp);
+
+        x.text = it->text;
+        x.layer = it->layer;
+        if (src->flip != dest->flip) {
+            flip_package_layer(x.layer);
+        }
+
+        x.size = it->size;
+        x.width = it->width;
+        dest->texts.push_back(&x);
+    }
+}
+
 void Board::smash_package_silkscreen_graphics(BoardPackage *pkg)
 {
+    if (pkg->omit_silkscreen)
+        return;
+
     std::map<Junction *, Junction *> junction_xlat;
     auto tr = pkg->placement;
     if (pkg->flip)
@@ -830,6 +961,55 @@ void Board::smash_package_silkscreen_graphics(BoardPackage *pkg)
     pkg->omit_silkscreen = true;
 }
 
+void Board::smash_panel_outline(BoardPanel &panel)
+{
+    if (panel.omit_outline)
+        return;
+
+    for (const auto &it : panel.included_board->board->polygons) {
+        if (it.second.layer == BoardLayers::L_OUTLINE) {
+            auto uu = UUID::random();
+            auto &new_poly = polygons.emplace(uu, uu).first->second;
+            new_poly.layer = BoardLayers::L_OUTLINE;
+            for (const auto &it_v : it.second.vertices) {
+                new_poly.vertices.emplace_back();
+                auto &v = new_poly.vertices.back();
+                v.arc_center = panel.placement.transform(it_v.arc_center);
+                v.arc_reverse = it_v.arc_reverse;
+                v.type = it_v.type;
+                v.position = panel.placement.transform(it_v.position);
+            }
+        }
+    }
+    panel.omit_outline = true;
+}
+
+void Board::smash_package_outline(BoardPackage &pkg)
+{
+    if (pkg.omit_outline)
+        return;
+    auto tr = pkg.placement;
+    if (pkg.flip)
+        tr.invert_angle();
+
+    for (const auto &it : pkg.package.polygons) {
+        if (it.second.layer == BoardLayers::L_OUTLINE) {
+            auto uu = UUID::random();
+            auto &new_poly = polygons.emplace(uu, uu).first->second;
+            new_poly.layer = BoardLayers::L_OUTLINE;
+            for (const auto &it_v : it.second.vertices) {
+                new_poly.vertices.emplace_back();
+                auto &v = new_poly.vertices.back();
+                v.arc_center = tr.transform(it_v.arc_center);
+                v.arc_reverse = it_v.arc_reverse;
+                v.type = it_v.type;
+                v.position = tr.transform(it_v.position);
+            }
+        }
+    }
+    pkg.omit_outline = true;
+}
+
 void Board::unsmash_package(BoardPackage *pkg)
 {
     if (!pkg->smashed)
@@ -906,6 +1086,30 @@ std::pair<Coordi, Coordi> Board::get_bbox() const
     return {a, b};
 }
 
+std::map<const BoardPackage *, PnPRow> Board::get_PnP(const PnPExportSettings &settings) const
+{
+    std::map<const BoardPackage *, PnPRow> r;
+    for (const auto &it : packages) {
+        if (it.second.component->nopopulate && !settings.include_nopopulate) {
+            continue;
+        }
+
+        PnPRow row;
+        row.refdes = it.second.component->refdes;
+        row.package = it.second.package.name;
+        row.MPN = it.second.component->part->get_MPN();
+        row.value = it.second.component->part->get_value();
+        row.manufacturer = it.second.component->part->get_manufacturer();
+        row.side = it.second.flip ? PnPRow::Side::BOTTOM : PnPRow::Side::TOP;
+        row.placement = it.second.placement;
+        if (it.second.flip)
+            row.placement.inc_angle_deg(180);
+        row.placement.mirror = false;
+        r.emplace(&it.second, row);
+    }
+    return r;
+}
+
 json Board::serialize() const
 {
     json j;
@@ -924,6 +1128,7 @@ json Board::serialize() const
     }
     j["pdf_export_settings"] = pdf_export_settings.serialize_board();
     j["step_export_settings"] = step_export_settings.serialize();
+    j["pnp_export_settings"] = pnp_export_settings.serialize();
     j["polygons"] = json::object();
     for (const auto &it : polygons) {
         j["polygons"][(std::string)it.first] = it.second.serialize();
@@ -980,6 +1185,18 @@ json Board::serialize() const
     for (const auto &it : connection_lines) {
         j["connection_lines"][(std::string)it.first] = it.second.serialize();
     }
+    if (included_boards.size()) {
+        j["included_boards"] = json::object();
+        for (const auto &it : included_boards) {
+            j["included_boards"][(std::string)it.first] = it.second.serialize();
+        }
+    }
+    if (board_panels.size()) {
+        j["board_panels"] = json::object();
+        for (const auto &it : board_panels) {
+            j["board_panels"][(std::string)it.first] = it.second.serialize();
+        }
+    }
     return j;
 }
 } // namespace horizon
