@@ -131,6 +131,8 @@ public:
     void set_appearance(const Appearance &a);
     const Color &get_color(ColorP colorp) const;
 
+    bool touchpad_pan = false;
+
     bool smooth_zoom = true;
     bool snap_to_targets = true;
 
@@ -149,6 +151,7 @@ public:
 
     CanvasAnnotation *create_annotation();
     void remove_annotation(CanvasAnnotation *a);
+    bool layer_is_annotation(int l) const;
 
 protected:
     void push() override;
@@ -236,7 +239,8 @@ private:
     Gdk::ModifierType grid_fine_modifier = Gdk::MOD1_MASK;
     float cursor_size = 20;
 
-    int annotation_layer_current = 20000;
+    static const int first_annotation_layer = 20000;
+    int annotation_layer_current = first_annotation_layer;
     std::map<int, CanvasAnnotation> annotations;
 
     void draw_bitmap_text(const Coordf &p, float scale, const std::string &rtext, int angle, ColorP color,
@@ -247,6 +251,19 @@ private:
 
     SelectionMode selection_mode = SelectionMode::HOVER;
 
+    Glib::RefPtr<Gtk::GestureZoom> gesture_zoom;
+    void zoom_gesture_begin_cb(GdkEventSequence *seq);
+    void zoom_gesture_update_cb(GdkEventSequence *seq);
+    Coord<float> gesture_zoom_pos_orig;
+    Coord<float> gesture_zoom_offset_orig;
+    float gesture_zoom_scale_orig = 1;
+
+    Glib::RefPtr<Gtk::GestureDrag> gesture_drag;
+    Coord<float> gesture_drag_offset_orig;
+
+    void drag_gesture_begin_cb(GdkEventSequence *seq);
+    void drag_gesture_update_cb(GdkEventSequence *seq);
+
 protected:
     void on_size_allocate(Gtk::Allocation &alloc) override;
     void on_realize() override;
