@@ -26,6 +26,8 @@
 #include "triangle.hpp"
 #include "util/placement.hpp"
 #include "util/util.hpp"
+#include "iairwire_filter.hpp"
+#include "board/board_panel.hpp"
 #include <algorithm>
 #include <ctime>
 #include <sstream>
@@ -207,13 +209,13 @@ ObjectRef Canvas::add_line(const std::deque<Coordi> &pts, int64_t width, ColorP
     return sr;
 }
 
-void Canvas::render(const Line &line, bool interactive)
+void Canvas::render(const Line &line, bool interactive, ColorP co)
 {
     img_line(line.from->position, line.to->position, line.width, line.layer);
     if (img_mode)
         return;
     triangle_type_current = Triangle::Type::GRAPHICS;
-    draw_line(line.from->position, line.to->position, ColorP::FROM_LAYER, line.layer, true, line.width);
+    draw_line(line.from->position, line.to->position, co, line.layer, true, line.width);
     triangle_type_current = Triangle::Type::NONE;
     if (interactive) {
         selectables.append_line(line.uuid, ObjectType::LINE, line.from->position, line.to->position, line.width, 0,
@@ -247,37 +249,31 @@ void Canvas::render(const LineNet &line)
     selectables.append_line(line.uuid, ObjectType::LINE_NET, line.from.get_position(), line.to.get_position(), width);
 }
 
-void Canvas::render(const Track &track)
+void Canvas::render(const Track &track, bool interactive)
 {
     auto c = ColorP::FROM_LAYER;
     if (track.net == nullptr) {
         c = ColorP::BUS;
     }
-    if (track.is_air) {
-        c = ColorP::AIRWIRE;
-    }
     auto width = track.width;
-    if (!track.is_air) {
-        img_net(track.net);
-        img_patch_type(PatchType::TRACK);
-        img_line(track.from.get_position(), track.to.get_position(), width, track.layer);
-        img_patch_type(PatchType::OTHER);
-        img_net(nullptr);
-    }
+    img_net(track.net);
+    img_patch_type(PatchType::TRACK);
+    img_line(track.from.get_position(), track.to.get_position(), width, track.layer);
+    img_patch_type(PatchType::OTHER);
+    img_net(nullptr);
     if (img_mode)
         return;
     auto layer = track.layer;
-    if (track.is_air)
-        layer = 10000;
 
     auto center = (track.from.get_position() + track.to.get_position()) / 2;
-    object_refs_current.emplace_back(ObjectType::TRACK, track.uuid);
+    if (interactive)
+        object_refs_current.emplace_back(ObjectType::TRACK, track.uuid);
     draw_line(track.from.get_position(), track.to.get_position(), c, layer, true, width);
-    if (track.locked && !track.is_air) {
+    if (track.locked) {
         auto ol = get_overlay_layer(layer);
         draw_lock(center, 0.7 * track.width, ColorP::TEXT_OVERLAY, ol, true);
     }
-    if (show_text_in_tracks && width > 0 && track.net && track.net->name.size() && track.from.is_junc()
+    if (interactive && show_text_in_tracks && width > 0 && track.net && track.net->name.size() && track.from.is_junc()
         && track.to.is_junc()) {
         auto overlay_layer = get_overlay_layer(track.layer, true);
         set_lod_size(width);
@@ -292,11 +288,16 @@ void Canvas::render(const Track &track)
         draw_bitmap_text_box(p, length, width, track.net->name, ColorP::TEXT_OVERLAY, overlay_layer, TextBoxMode::FULL);
         set_lod_size(-1);
     }
-    object_refs_current.pop_back();
-    if (!track.is_air) {
+    if (interactive)
+        object_refs_current.pop_back();
+    if (interactive)
         selectables.append_line(track.uuid, ObjectType::TRACK, track.from.get_position(), track.to.get_position(),
                                 track.width, 0, track.layer);
-    }
+}
+
+void Canvas::render(const Airwire &airwire)
+{
+    draw_line(airwire.from.get_position(), airwire.to.get_position(), ColorP::AIRWIRE, 10000, true, 0);
 }
 
 void Canvas::render(const ConnectionLine &line)
@@ -308,7 +309,7 @@ void Canvas::render(const ConnectionLine &line)
                             0);
 }
 
-void Canvas::render(const SymbolPin &pin, bool interactive)
+void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
 {
     Coordi p0 = transform.transform(pin.position);
     Coordi p1 = p0;
@@ -395,14 +396,22 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
         pad_orientation = Orientation::DOWN;
         break;
     }
+    ColorP c_main = co;
     ColorP c_name = ColorP::PIN;
     ColorP c_pad = ColorP::PIN;
+    ColorP c_pin = ColorP::PIN;
+
     if (!pin.name_visible) {
         c_name = ColorP::PIN_HIDDEN;
     }
     if (!pin.pad_visible) {
         c_pad = ColorP::PIN_HIDDEN;
     }
+    if (co != ColorP::FROM_LAYER) {
+        c_name = co;
+        c_pad = co;
+        c_pin = co;
+    }
     img_auto_line = img_mode;
     if (interactive || pin.name_visible) {
         bool draw_in_line = pin.name_orientation == SymbolPin::NameOrientation::IN_LINE
@@ -447,18 +456,15 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
     }
 
     if (pin.decoration.dot) {
-        draw_arc(Coordf(-((int64_t)pin.length) + dot_size / 2, 0), dot_size / 2, 0, 2 * M_PI, ColorP::FROM_LAYER, 0,
-                 true, 0);
+        draw_arc(Coordf(-((int64_t)pin.length) + dot_size / 2, 0), dot_size / 2, 0, 2 * M_PI, c_main, 0, true, 0);
     }
     if (pin.decoration.clock) {
-        draw_line(Coordf(-(int64_t)pin.length, .375_mm), Coordf(-(int64_t)pin.length - .75_mm, 0), ColorP::FROM_LAYER,
-                  0, true, 0);
-        draw_line(Coordf(-(int64_t)pin.length, -.375_mm), Coordf(-(int64_t)pin.length - .75_mm, 0), ColorP::FROM_LAYER,
-                  0, true, 0);
+        draw_line(Coordf(-(int64_t)pin.length, .375_mm), Coordf(-(int64_t)pin.length - .75_mm, 0), c_main, 0, true, 0);
+        draw_line(Coordf(-(int64_t)pin.length, -.375_mm), Coordf(-(int64_t)pin.length - .75_mm, 0), c_main, 0, true, 0);
     }
     {
-        auto dl = [this](float ax, float ay, float bx, float by) {
-            draw_line(Coordf(ax * 1_mm, ay * 1_mm), Coordf(bx * 1_mm, by * 1_mm), ColorP::PIN, 0, true, 0);
+        auto dl = [this, c_pin](float ax, float ay, float bx, float by) {
+            draw_line(Coordf(ax * 1_mm, ay * 1_mm), Coordf(bx * 1_mm, by * 1_mm), c_pin, 0, true, 0);
         };
         switch (pin.direction) {
         case Pin::Direction::OUTPUT:
@@ -497,9 +503,9 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
     transform_restore();
 
     if (pin.decoration.schmitt) {
-        auto dl = [this](float ax, float ay, float bx, float by) {
+        auto dl = [this, c_pin](float ax, float ay, float bx, float by) {
             auto sc = .025_mm;
-            draw_line(Coordf(ax * sc, ay * sc), Coordf(bx * sc, by * sc), ColorP::PIN, 0, true, 0);
+            draw_line(Coordf(ax * sc, ay * sc), Coordf(bx * sc, by * sc), c_pin, 0, true, 0);
         };
         transform_save();
         transform.accumulate(pin.position);
@@ -514,9 +520,9 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
         transform_restore();
     }
     if (pin.decoration.driver != SymbolPin::Decoration::Driver::DEFAULT) {
-        auto dl = [this](float ax, float ay, float bx, float by) {
+        auto dl = [this, c_pin](float ax, float ay, float bx, float by) {
             auto sc = .5_mm;
-            draw_line(Coordf(ax * sc, ay * sc), Coordf(bx * sc, by * sc), ColorP::PIN, 0, true, 0);
+            draw_line(Coordf(ax * sc, ay * sc), Coordf(bx * sc, by * sc), c_pin, 0, true, 0);
         };
         transform_save();
         transform.accumulate(pin.position);
@@ -560,21 +566,21 @@ void Canvas::render(const SymbolPin &pin, bool interactive)
 
     switch (pin.connector_style) {
     case SymbolPin::ConnectorStyle::BOX:
-        draw_box(p0, 0.25_mm, ColorP::FROM_LAYER, 0, false);
+        draw_box(p0, 0.25_mm, c_main, 0, false);
         break;
 
     case SymbolPin::ConnectorStyle::NC:
-        draw_cross(p0, 0.25_mm, ColorP::FROM_LAYER, 0, false);
-        draw_text0(p_nc, 1.5_mm, "NC", orientation_to_angle(nc_orientation), false, TextOrigin::CENTER, ColorP::PIN, 0);
+        draw_cross(p0, 0.25_mm, c_main, 0, false);
+        draw_text0(p_nc, 1.5_mm, "NC", orientation_to_angle(nc_orientation), false, TextOrigin::CENTER, c_pin, 0);
         break;
 
     case SymbolPin::ConnectorStyle::NONE:
         break;
     }
     if (pin.connected_net_lines.size() > 1) {
-        draw_line(p0, p0 + Coordi(0, 10), ColorP::FROM_LAYER, 0, false, 0.75_mm);
+        draw_line(p0, p0 + Coordi(0, 10), c_main, 0, false, 0.75_mm);
     }
-    draw_line(p0, p1, ColorP::FROM_LAYER, 0, false);
+    draw_line(p0, p1, c_main, 0, false);
     if (interactive)
         selectables.append(pin.uuid, ObjectType::SYMBOL_PIN, p0, Coordf::min(pad_extents.first, Coordf::min(p0, p1)),
                            Coordf::max(pad_extents.second, Coordf::max(p0, p1)));
@@ -586,7 +592,7 @@ static int64_t sq(int64_t x)
     return x * x;
 }
 
-void Canvas::render(const Arc &arc, bool interactive)
+void Canvas::render(const Arc &arc, bool interactive, ColorP co)
 {
     Coordf a(arc.from->position);   // ,b,c;
     Coordf b(arc.to->position);     // ,b,c;
@@ -595,7 +601,7 @@ void Canvas::render(const Arc &arc, bool interactive)
     float radius1 = sqrt(sq(c.x - b.x) + sq(c.y - b.y));
     float a0 = atan2f(a.y - c.y, a.x - c.x);
     float a1 = atan2f(b.y - c.y, b.x - c.x);
-    auto bb = draw_arc2(c, radius0, a0, radius1, a1, ColorP::FROM_LAYER, arc.layer, true, arc.width);
+    auto bb = draw_arc2(c, radius0, a0, radius1, a1, co, arc.layer, true, arc.width);
     Coordf t(radius0, radius0);
     if (interactive)
         selectables.append(arc.uuid, ObjectType::ARC, c, bb.first, bb.second, 0, arc.layer);
@@ -605,7 +611,7 @@ void Canvas::render(const SchematicSymbol &sym)
 {
     transform = sym.placement;
     object_refs_current.emplace_back(ObjectType::SCHEMATIC_SYMBOL, sym.uuid);
-    render(sym.symbol, true, sym.smashed);
+    render(sym.symbol, true, sym.smashed, ColorP::FROM_LAYER);
     object_refs_current.pop_back();
     for (const auto &it : sym.symbol.pins) {
         targets.emplace_back(UUIDPath<2>(sym.uuid, it.second.uuid), ObjectType::SYMBOL_PIN,
@@ -614,9 +620,24 @@ void Canvas::render(const SchematicSymbol &sym)
     auto bb = sym.symbol.get_bbox();
     selectables.append(sym.uuid, ObjectType::SCHEMATIC_SYMBOL, {0, 0}, bb.first, bb.second);
     transform.reset();
+
+    for (const auto &it : sym.texts) {
+        render(*it, true, ColorP::FROM_LAYER);
+    }
+
+    if (sym.component->nopopulate) {
+        transform = sym.placement;
+        img_auto_line = img_mode;
+        draw_line(bb.first - Coordi(0.2_mm, 0.2_mm), bb.second + Coordi(0.2_mm, 0.2_mm), ColorP::NOPOPULATE_X, 0, true,
+                  0.2_mm);
+        draw_line(Coordi(bb.first.x, bb.second.y) + Coordi(-0.2_mm, 0.2_mm),
+                  Coordi(bb.second.x, bb.first.y) + Coordi(0.2_mm, -0.2_mm), ColorP::NOPOPULATE_X, 0, true, 0.2_mm);
+        img_auto_line = false;
+        transform.reset();
+    }
 }
 
-void Canvas::render(const Text &text, bool interactive)
+void Canvas::render(const Text &text, bool interactive, ColorP co)
 {
     bool rev = layer_provider->get_layers().at(text.layer).reverse;
     transform_save();
@@ -629,8 +650,7 @@ void Canvas::render(const Text &text, bool interactive)
     img_patch_type(PatchType::TEXT);
     triangle_type_current = Triangle::Type::TEXT;
     auto extents = draw_text0(transform.shift, text.size, text.overridden ? text.text_override : text.text, angle, rev,
-                              text.origin, ColorP::FROM_LAYER, text.layer, text.width, true, text.font, false,
-                              transform.mirror);
+                              text.origin, co, text.layer, text.width, true, text.font, false, transform.mirror);
     triangle_type_current = Triangle::Type::NONE;
     // img_text_extents(text, extents);
     img_patch_type(PatchType::OTHER);
@@ -743,7 +763,7 @@ static const Coordf coordf_from_pt(const TPPLPoint &p)
     return Coordf(p.x, p.y);
 }
 
-void Canvas::render(const Polygon &ipoly, bool interactive)
+void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
 {
     img_polygon(ipoly);
     if (img_mode)
@@ -758,25 +778,26 @@ void Canvas::render(const Polygon &ipoly, bool interactive)
         triangle_type_current = Triangle::Type::PLANE;
         auto tris = fragment_cache.get_triangles(*plane);
         for (const auto &tri : tris) {
-            add_triangle(poly.layer, tri[0], tri[1], tri[2], ColorP::FROM_LAYER);
+            add_triangle(poly.layer, transform.transform(tri[0]), transform.transform(tri[1]),
+                         transform.transform(tri[2]), co);
         }
         for (const auto &frag : plane->fragments) {
-            ColorP co = ColorP::FROM_LAYER;
+            ColorP co_orphan = co;
             if (frag.orphan == true)
-                co = ColorP::FRAG_ORPHAN;
+                co_orphan = ColorP::FRAG_ORPHAN;
 
             for (const auto &path : frag.paths) {
                 for (size_t i = 0; i < path.size(); i++) {
                     auto &c0 = path[i];
                     auto &c1 = path[(i + 1) % path.size()];
-                    draw_line(Coordf(c0.X, c0.Y), Coordf(c1.X, c1.Y), co, poly.layer);
+                    draw_line(Coordf(c0.X, c0.Y), Coordf(c1.X, c1.Y), co_orphan, poly.layer);
                 }
             }
         }
         if (plane->fragments.size() == 0) { // empty, draw poly outline
             for (size_t i = 0; i < poly.vertices.size(); i++) {
-                draw_line(poly.vertices[i].position, poly.vertices[(i + 1) % poly.vertices.size()].position,
-                          ColorP::FROM_LAYER, poly.layer);
+                draw_line(poly.vertices[i].position, poly.vertices[(i + 1) % poly.vertices.size()].position, co,
+                          poly.layer);
             }
         }
         triangle_type_current = Triangle::Type::NONE;
@@ -803,11 +824,11 @@ void Canvas::render(const Polygon &ipoly, bool interactive)
             Coordf p0 = transform.transform(coordf_from_pt(tri[0]));
             Coordf p1 = transform.transform(coordf_from_pt(tri[1]));
             Coordf p2 = transform.transform(coordf_from_pt(tri[2]));
-            add_triangle(poly.layer, p0, p1, p2, ColorP::FROM_LAYER);
+            add_triangle(poly.layer, p0, p1, p2, co);
         }
         for (size_t i = 0; i < poly.vertices.size(); i++) {
-            draw_line(poly.vertices[i].position, poly.vertices[(i + 1) % poly.vertices.size()].position,
-                      ColorP::FROM_LAYER, poly.layer);
+            draw_line(poly.vertices[i].position, poly.vertices[(i + 1) % poly.vertices.size()].position, co,
+                      poly.layer);
         }
         triangle_type_current = Triangle::Type::NONE;
     }
@@ -935,7 +956,7 @@ void Canvas::render(const Pad &pad)
     transform_restore();
 }
 
-void Canvas::render(const Symbol &sym, bool on_sheet, bool smashed)
+void Canvas::render(const Symbol &sym, bool on_sheet, bool smashed, ColorP co)
 {
     if (!on_sheet) {
         for (const auto &it : sym.junctions) {
@@ -946,33 +967,34 @@ void Canvas::render(const Symbol &sym, bool on_sheet, bool smashed)
             }
         }
     }
+
     for (const auto &it : sym.lines) {
-        render(it.second, !on_sheet);
+        render(it.second, !on_sheet, co);
     }
 
     if (object_refs_current.size() && object_refs_current.back().type == ObjectType::SCHEMATIC_SYMBOL) {
         auto sym_uuid = object_refs_current.back().uuid;
         for (const auto &it : sym.pins) {
             object_refs_current.emplace_back(ObjectType::SYMBOL_PIN, it.second.uuid, sym_uuid);
-            render(it.second, !on_sheet);
+            render(it.second, !on_sheet, co);
             object_refs_current.pop_back();
         }
     }
     else {
         for (const auto &it : sym.pins) {
-            render(it.second, !on_sheet);
+            render(it.second, !on_sheet, co);
         }
     }
 
     for (const auto &it : sym.arcs) {
-        render(it.second, !on_sheet);
+        render(it.second, !on_sheet, co);
     }
     for (const auto &it : sym.polygons) {
-        render(it.second, !on_sheet);
+        render(it.second, !on_sheet, co);
     }
     if (!smashed) {
         for (const auto &it : sym.texts) {
-            render(it.second, !on_sheet);
+            render(it.second, !on_sheet, co);
         }
     }
 }
@@ -992,7 +1014,8 @@ void Canvas::render(const Sheet &sheet)
         render(it.second);
     }
     for (const auto &it : sheet.texts) {
-        render(it.second);
+        if (!it.second.from_smash)
+            render(it.second);
     }
     for (const auto &it : sheet.net_labels) {
         render(it.second);
@@ -1084,14 +1107,15 @@ void Canvas::render_pad_overlay(const Pad &pad)
     transform_restore();
 }
 
-void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omit_silkscreen)
+void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omit_silkscreen, bool omit_outline)
 {
     if (interactive) {
         for (const auto &it : pkg.junctions) {
             auto &junc = it.second;
-            selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, 10000, true);
+            selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, junc.layer);
             if (!junc.temp) {
-                targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position));
+                targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position), 0,
+                                     junc.layer);
             }
         }
     }
@@ -1122,7 +1146,7 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omi
             object_refs_current.pop_back();
         }
     }
-    else {
+    else if (interactive) {
         for (const auto &it : pkg.pads) {
             object_refs_current.emplace_back(ObjectType::PAD, it.second.uuid);
             render_pad_overlay(it.second);
@@ -1130,9 +1154,17 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omi
             object_refs_current.pop_back();
         }
     }
+    else {
+        for (const auto &it : pkg.pads) {
+            render_pad_overlay(it.second);
+            render(it.second);
+        }
+    }
     for (const auto &it : pkg.polygons) {
         if (omit_silkscreen && BoardLayers::is_silkscreen(it.second.layer))
             continue;
+        if (omit_outline && it.second.layer == BoardLayers::L_OUTLINE)
+            continue;
         render(it.second, interactive);
     }
 
@@ -1191,63 +1223,81 @@ void Canvas::render(const Buffer &buf)
     }
 }
 
-void Canvas::render(const BoardPackage &pkg)
+void Canvas::render(const BoardPackage &pkg, bool interactive)
 {
-    transform = pkg.placement;
+    transform_save();
+    transform.accumulate(pkg.placement);
     auto bb = pkg.package.get_bbox();
     if (pkg.flip) {
         transform.invert_angle();
     }
-    selectables.append(pkg.uuid, ObjectType::BOARD_PACKAGE, {0, 0}, bb.first, bb.second, 0,
-                       pkg.flip ? BoardLayers::BOTTOM_PACKAGE : BoardLayers::TOP_PACKAGE);
-    targets.emplace_back(pkg.uuid, ObjectType::BOARD_PACKAGE, pkg.placement.shift);
-    for (const auto &it : pkg.package.pads) {
-        targets.emplace_back(UUIDPath<2>(pkg.uuid, it.first), ObjectType::PAD,
-                             transform.transform(it.second.placement.shift));
+    if (interactive) {
+        selectables.append(pkg.uuid, ObjectType::BOARD_PACKAGE, {0, 0}, bb.first, bb.second, 0,
+                           pkg.flip ? BoardLayers::BOTTOM_PACKAGE : BoardLayers::TOP_PACKAGE);
+        targets.emplace_back(pkg.uuid, ObjectType::BOARD_PACKAGE, pkg.placement.shift);
+
+        for (const auto &it : pkg.package.pads) {
+            targets.emplace_back(UUIDPath<2>(pkg.uuid, it.first), ObjectType::PAD,
+                                 transform.transform(it.second.placement.shift));
+        }
     }
-    object_refs_current.emplace_back(ObjectType::BOARD_PACKAGE, pkg.uuid);
-    render(pkg.package, false, pkg.smashed, pkg.omit_silkscreen);
-    object_refs_current.pop_back();
+    if (interactive)
+        object_refs_current.emplace_back(ObjectType::BOARD_PACKAGE, pkg.uuid);
+
+    render(pkg.package, false, pkg.smashed, pkg.omit_silkscreen, pkg.omit_outline);
+
+    if (interactive)
+        object_refs_current.pop_back();
 
     transform.reset();
+    transform_restore();
 }
 
-void Canvas::render(const Via &via)
+void Canvas::render(const Via &via, bool interactive)
 {
     transform_save();
-    transform.reset();
-    transform.shift = via.junction->position;
+    {
+        Placement pl;
+        pl.shift = via.junction->position;
+        transform.accumulate(pl);
+    }
     auto bb = via.padstack.get_bbox();
-    selectables.append(via.uuid, ObjectType::VIA, {0, 0}, bb.first, bb.second);
+    if (interactive)
+        selectables.append(via.uuid, ObjectType::VIA, {0, 0}, bb.first, bb.second);
     img_net(via.junction->net);
     img_patch_type(PatchType::VIA);
-    object_refs_current.emplace_back(ObjectType::VIA, via.uuid);
+    if (interactive)
+        object_refs_current.emplace_back(ObjectType::VIA, via.uuid);
     render(via.padstack, false);
     if (via.locked) {
         auto ol = get_overlay_layer(BoardLayers::TOP_COPPER);
         draw_lock({0, 0}, 0.7 * std::min(std::abs(bb.second.x - bb.first.x), std::abs(bb.second.y - bb.first.y)),
                   ColorP::TEXT_OVERLAY, ol, true);
     }
-    object_refs_current.pop_back();
+    if (interactive)
+        object_refs_current.pop_back();
     img_net(nullptr);
     img_patch_type(PatchType::OTHER);
     transform_restore();
 }
 
-void Canvas::render(const BoardHole &hole)
+void Canvas::render(const BoardHole &hole, bool interactive)
 {
     transform_save();
-    transform = hole.placement;
+    transform.accumulate(hole.placement);
     auto bb = hole.padstack.get_bbox();
-    selectables.append(hole.uuid, ObjectType::BOARD_HOLE, {0, 0}, bb.first, bb.second);
+    if (interactive)
+        selectables.append(hole.uuid, ObjectType::BOARD_HOLE, {0, 0}, bb.first, bb.second);
     img_net(hole.net);
     if (hole.padstack.type == Padstack::Type::HOLE)
         img_patch_type(PatchType::HOLE_PTH);
     else
         img_patch_type(PatchType::HOLE_NPTH);
-    object_refs_current.emplace_back(ObjectType::BOARD_HOLE, hole.uuid);
+    if (interactive)
+        object_refs_current.emplace_back(ObjectType::BOARD_HOLE, hole.uuid);
     render(hole.padstack, false);
-    object_refs_current.pop_back();
+    if (interactive)
+        object_refs_current.pop_back();
     img_net(nullptr);
     img_patch_type(PatchType::OTHER);
     transform_restore();
@@ -1346,69 +1396,89 @@ void Canvas::render(const class Dimension &dim)
     }
 }
 
-void Canvas::render(const Board &brd)
+void Canvas::render(const Board &brd, bool interactive, PanelMode mode, OutlineMode outline_mode)
 {
     clock_t begin = clock();
 
     for (const auto &it : brd.holes) {
-        render(it.second);
+        render(it.second, interactive);
     }
-    for (const auto &it : brd.junctions) {
-        render(it.second, true, ObjectType::BOARD);
+    if (interactive) {
+        for (const auto &it : brd.junctions) {
+            render(it.second, true, ObjectType::BOARD);
+        }
     }
-    for (const auto &it : brd.airwires) {
-        render(it.second);
-    }
-    for (const auto &it : brd.polygons) {
-        render(it.second);
-    }
-    for (const auto &it : brd.texts) {
-        render(it.second);
-    }
-    for (const auto &it : brd.tracks) {
-        render(it.second);
-    }
-    for (const auto &it : brd.packages) {
-        render(it.second);
-    }
-    for (const auto &it : brd.vias) {
-        render(it.second);
-    }
-    for (const auto &it : brd.lines) {
-        render(it.second);
-    }
-    for (const auto &it : brd.arcs) {
-        render(it.second);
-    }
-    for (const auto &it : brd.dimensions) {
-        render(it.second);
-    }
-    for (const auto &it : brd.connection_lines) {
-        render(it.second);
-    }
-
-    for (const auto &path : brd.obstacles) {
-        for (auto it = path.cbegin(); it < path.cend(); it++) {
-            if (it != path.cbegin()) {
-                auto b = it - 1;
-                draw_line(Coordf(b->X, b->Y), Coordf(it->X, it->Y), ColorP::AIRWIRE, 10000, false, 0);
+    if (!img_mode && interactive) {
+        for (const auto &it_net : brd.airwires) {
+            if (airwire_filter == nullptr || airwire_filter->airwire_is_visible(it_net.first)) {
+                object_refs_current.emplace_back(ObjectType::AIRWIRE, it_net.first);
+                for (const auto &it : it_net.second) {
+                    render(it);
+                }
+                object_refs_current.pop_back();
             }
         }
     }
-
-    unsigned int i = 0;
-    for (auto it = brd.track_path.cbegin(); it < brd.track_path.cend(); it++) {
-        if (it != brd.track_path.cbegin()) {
-            auto b = it - 1;
-            draw_line(Coordf(b->X, b->Y), Coordf(it->X, it->Y), ColorP::AIRWIRE, 10000, false, 0);
-        }
-        if (i % 2 == 0) {
-            draw_line(Coordf(it->X, it->Y), Coordf(it->X, it->Y), ColorP::AIRWIRE, 10000, false, .1_mm);
-        }
-        i++;
+    for (const auto &it : brd.polygons) {
+        if (outline_mode == OutlineMode::OMIT && it.second.layer == BoardLayers::L_OUTLINE)
+            continue;
+        render(it.second, interactive);
     }
-    for (const auto &it : brd.warnings) {
-        render(it);
+    for (const auto &it : brd.texts) {
+        render(it.second, interactive);
+    }
+    for (const auto &it : brd.tracks) {
+        render(it.second, interactive);
+    }
+    for (const auto &it : brd.packages) {
+        render(it.second, interactive);
+    }
+    for (const auto &it : brd.vias) {
+        render(it.second, interactive);
+    }
+    for (const auto &it : brd.lines) {
+        render(it.second, interactive);
+    }
+    for (const auto &it : brd.arcs) {
+        render(it.second, interactive);
+    }
+    if (mode == PanelMode::INCLUDE) {
+        for (const auto &it : brd.board_panels) {
+            render(it.second);
+        }
+    }
+
+    if (interactive) {
+        for (const auto &it : brd.dimensions) {
+            render(it.second);
+        }
+        for (const auto &it : brd.connection_lines) {
+            render(it.second);
+        }
+
+        for (const auto &path : brd.obstacles) {
+            for (auto it = path.cbegin(); it < path.cend(); it++) {
+                if (it != path.cbegin()) {
+                    auto b = it - 1;
+                    draw_line(Coordf(b->X, b->Y), Coordf(it->X, it->Y), ColorP::AIRWIRE, 10000, false, 0);
+                }
+            }
+        }
+
+        unsigned int i = 0;
+        for (auto it = brd.track_path.cbegin(); it < brd.track_path.cend(); it++) {
+            if (it != brd.track_path.cbegin()) {
+                auto b = it - 1;
+                draw_line(Coordf(b->X, b->Y), Coordf(it->X, it->Y), ColorP::AIRWIRE, 10000, false, 0);
+            }
+            if (i % 2 == 0) {
+                draw_line(Coordf(it->X, it->Y), Coordf(it->X, it->Y), ColorP::AIRWIRE, 10000, false, .1_mm);
+            }
+            i++;
+        }
+        for (const auto &it : brd.warnings) {
+            render(it);
+        }
     }
 
     clock_t end = clock();
@@ -1416,6 +1486,21 @@ void Canvas::render(const Board &brd)
     std::cout << "render took " << 1 / elapsed_secs << std::endl;
 }
 
+void Canvas::render(const BoardPanel &panel)
+{
+    if (!panel.included_board->is_valid()) {
+        draw_error(panel.placement.shift, 2e5, "invalid board", false);
+        return;
+    }
+    transform_save();
+    transform.accumulate(panel.placement);
+    auto bb = panel.included_board->board->get_bbox();
+    selectables.append(panel.uuid, ObjectType::BOARD_PANEL, {0, 0}, bb.first, bb.second, 0, 10000);
+    render(*panel.included_board->board, false, PanelMode::SKIP,
+           panel.omit_outline ? OutlineMode::OMIT : OutlineMode::INCLUDE);
+    transform_restore();
+}
+
 void Canvas::render(const Frame &fr, bool on_sheet)
 {
     if (!on_sheet) {
