@@ -1,14 +1,14 @@
 #include "cover.hpp"
 #include "board/board_layers.hpp"
 #include "canvas/gl_util.hpp"
-#include "canvas3d.hpp"
+#include "canvas3d_base.hpp"
 #include <cmath>
 #include <glm/glm.hpp>
 #include <glm/gtc/matrix_transform.hpp>
 #include <glm/gtc/type_ptr.hpp>
 
 namespace horizon {
-CoverRenderer::CoverRenderer(Canvas3D *c) : ca(c)
+CoverRenderer::CoverRenderer(Canvas3DBase &c) : ca(c)
 {
 }
 
@@ -25,7 +25,7 @@ static GLuint create_vao(GLuint program, GLuint &vbo_out)
     glGenBuffers(1, &buffer);
     glBindBuffer(GL_ARRAY_BUFFER, buffer);
 
-    Canvas3D::Layer3D::Vertex vertices[] = {
+    CanvasMesh::Layer3D::Vertex vertices[] = {
             //   Position
             {-5, -5}, {5, -5}, {-5, 5},
 
@@ -35,7 +35,7 @@ static GLuint create_vao(GLuint program, GLuint &vbo_out)
 
     /* enable and set the position attribute */
     glEnableVertexAttribArray(position_index);
-    glVertexAttribPointer(position_index, 2, GL_FLOAT, GL_FALSE, sizeof(Canvas3D::Layer3D::Vertex), 0);
+    glVertexAttribPointer(position_index, 2, GL_FLOAT, GL_FALSE, sizeof(CanvasMesh::Layer3D::Vertex), 0);
 
     /* enable and set the color attribute */
     /* reset the state; we will re-enable the VAO when needed */
@@ -69,16 +69,16 @@ void CoverRenderer::push()
 {
     glBindBuffer(GL_ARRAY_BUFFER, vbo);
     n_vertices = 0;
-    for (const auto &it : ca->layers) {
+    for (const auto &it : ca.ca.get_layers()) {
         n_vertices += it.second.tris.size();
     }
-    glBufferData(GL_ARRAY_BUFFER, sizeof(Canvas3D::Layer3D::Vertex) * n_vertices, nullptr, GL_STREAM_DRAW);
+    glBufferData(GL_ARRAY_BUFFER, sizeof(CanvasMesh::Layer3D::Vertex) * n_vertices, nullptr, GL_STREAM_DRAW);
     GL_CHECK_ERROR
     size_t ofs = 0;
     layer_offsets.clear();
-    for (const auto &it : ca->layers) {
-        glBufferSubData(GL_ARRAY_BUFFER, ofs * sizeof(Canvas3D::Layer3D::Vertex),
-                        it.second.tris.size() * sizeof(Canvas3D::Layer3D::Vertex), it.second.tris.data());
+    for (const auto &it : ca.ca.get_layers()) {
+        glBufferSubData(GL_ARRAY_BUFFER, ofs * sizeof(CanvasMesh::Layer3D::Vertex),
+                        it.second.tris.size() * sizeof(CanvasMesh::Layer3D::Vertex), it.second.tris.data());
         layer_offsets[it.first] = ofs;
         ofs += it.second.tris.size();
     }
@@ -87,13 +87,13 @@ void CoverRenderer::push()
 
 void CoverRenderer::render(int layer)
 {
-    if (ca->layers[layer].alpha != 1)
+    if (ca.ca.get_layer(layer).alpha != 1)
         glEnable(GL_BLEND);
-    auto co = ca->get_layer_color(layer);
-    glUniform4f(layer_color_loc, co.r, co.g, co.b, ca->layers[layer].alpha);
-    glUniform1f(layer_offset_loc, ca->get_layer_offset(layer));
-    glUniform1f(layer_thickness_loc, ca->get_layer_thickness(layer));
-    glDrawArrays(GL_TRIANGLES, layer_offsets[layer], ca->layers[layer].tris.size());
+    auto co = ca.get_layer_color(layer);
+    gl_color_to_uniform_4f(layer_color_loc, co, ca.ca.get_layer(layer).alpha);
+    glUniform1f(layer_offset_loc, ca.get_layer_offset(layer));
+    glUniform1f(layer_thickness_loc, ca.get_layer_thickness(layer));
+    glDrawArrays(GL_TRIANGLES, layer_offsets[layer], ca.ca.get_layer(layer).tris.size());
     glDisable(GL_BLEND);
 }
 
@@ -102,16 +102,16 @@ void CoverRenderer::render()
     glUseProgram(program);
     glBindVertexArray(vao);
 
-    glUniformMatrix4fv(view_loc, 1, GL_FALSE, glm::value_ptr(ca->viewmat));
-    glUniformMatrix4fv(proj_loc, 1, GL_FALSE, glm::value_ptr(ca->projmat));
-    glUniform3fv(cam_normal_loc, 1, glm::value_ptr(ca->cam_normal));
+    glUniformMatrix4fv(view_loc, 1, GL_FALSE, glm::value_ptr(ca.viewmat));
+    glUniformMatrix4fv(proj_loc, 1, GL_FALSE, glm::value_ptr(ca.projmat));
+    glUniform3fv(cam_normal_loc, 1, glm::value_ptr(ca.cam_normal));
 
     for (const auto &it : layer_offsets) {
-        if (ca->layers[it.first].alpha == 1 && ca->layer_is_visible(it.first))
+        if (ca.ca.get_layer(it.first).alpha == 1 && ca.layer_is_visible(it.first))
             render(it.first);
     }
     for (const auto &it : layer_offsets) {
-        if (ca->layers[it.first].alpha != 1 && ca->layer_is_visible(it.first))
+        if (ca.ca.get_layer(it.first).alpha != 1 && ca.layer_is_visible(it.first))
             render(it.first);
     }
 }
