@@ -4,31 +4,30 @@
 #include <iostream>
 #include <memory>
 #include "nlohmann/json.hpp"
+#include "document/idocument_frame.hpp"
 
 namespace horizon {
-class CoreFrame : public Core {
+class CoreFrame : public Core, public IDocumentFrame {
 public:
     CoreFrame(const std::string &frame_filename);
     bool has_object_type(ObjectType ty) const override;
 
-    Frame *get_frame();
+    Frame *get_frame() override;
     const Frame *get_canvas_data() const;
     class LayerProvider *get_layer_provider() override;
 
     void rebuild(bool from_undo = false) override;
-    void commit() override;
-    void revert() override;
 
     std::pair<Coordi, Coordi> get_bbox() override;
 
     const std::string &get_filename() const override;
 
 private:
-    std::map<UUID, Polygon> *get_polygon_map(bool work = true) override;
-    std::map<UUID, Junction> *get_junction_map(bool work = true) override;
-    std::map<UUID, Text> *get_text_map(bool work = true) override;
-    std::map<UUID, Line> *get_line_map(bool work = true) override;
-    std::map<UUID, Arc> *get_arc_map(bool work = true) override;
+    std::map<UUID, Polygon> *get_polygon_map() override;
+    std::map<UUID, Junction> *get_junction_map() override;
+    std::map<UUID, Text> *get_text_map() override;
+    std::map<UUID, Line> *get_line_map() override;
+    std::map<UUID, Arc> *get_arc_map() override;
 
     Frame frame;
 
