@@ -9,8 +9,9 @@
 
 namespace horizon {
 CoreSchematic::CoreSchematic(const std::string &schematic_filename, const std::string &block_filename, Pool &pool)
-    : block(Block::new_from_file(block_filename, pool)), sch(Schematic::new_from_file(schematic_filename, block, pool)),
-      rules(sch.rules), bom_export_settings(block.bom_export_settings), pdf_export_settings(sch.pdf_export_settings),
+    : block(Block::new_from_file(block_filename, pool)), project_meta_loaded_from_block(block.project_meta.size()),
+      sch(Schematic::new_from_file(schematic_filename, block, pool)), rules(sch.rules),
+      bom_export_settings(block.bom_export_settings), pdf_export_settings(sch.pdf_export_settings),
       m_schematic_filename(schematic_filename), m_block_filename(block_filename)
 {
     auto x = std::find_if(sch.sheets.cbegin(), sch.sheets.cend(), [](const auto &a) { return a.second.index == 1; });
@@ -20,29 +21,29 @@ CoreSchematic::CoreSchematic(const std::string &schematic_filename, const std::s
     rebuild();
 }
 
-Junction *CoreSchematic::get_junction(const UUID &uu, bool work)
+Junction *CoreSchematic::get_junction(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     return &sheet.junctions.at(uu);
 }
-SchematicSymbol *CoreSchematic::get_schematic_symbol(const UUID &uu, bool work)
+SchematicSymbol *CoreSchematic::get_schematic_symbol(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     return &sheet.symbols.at(uu);
 }
-Text *CoreSchematic::get_text(const UUID &uu, bool work)
+Text *CoreSchematic::get_text(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     return &sheet.texts.at(uu);
 }
-Schematic *CoreSchematic::get_schematic(bool work)
+Schematic *CoreSchematic::get_schematic()
 {
     return &sch;
 }
 
-Block *CoreSchematic::get_block(bool work)
+Block *CoreSchematic::get_block()
 {
-    return get_schematic(work)->block;
+    return get_schematic()->block;
 }
 
 LayerProvider *CoreSchematic::get_layer_provider()
@@ -50,51 +51,51 @@ LayerProvider *CoreSchematic::get_layer_provider()
     return get_sheet();
 }
 
-Sheet *CoreSchematic::get_sheet(bool work)
+Sheet *CoreSchematic::get_sheet()
 {
     return &sch.sheets.at(sheet_uuid);
 }
-Line *CoreSchematic::get_line(const UUID &uu, bool work)
+Line *CoreSchematic::get_line(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     return &sheet.lines.at(uu);
 }
-Arc *CoreSchematic::get_arc(const UUID &uu, bool work)
+Arc *CoreSchematic::get_arc(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     return &sheet.arcs.at(uu);
 }
 
-Junction *CoreSchematic::insert_junction(const UUID &uu, bool work)
+Junction *CoreSchematic::insert_junction(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     auto x = sheet.junctions.emplace(std::make_pair(uu, uu));
     return &(x.first->second);
 }
 
-LineNet *CoreSchematic::insert_line_net(const UUID &uu, bool work)
+LineNet *CoreSchematic::insert_line_net(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     auto x = sheet.net_lines.emplace(std::make_pair(uu, uu));
     return &(x.first->second);
 }
 
-void CoreSchematic::delete_junction(const UUID &uu, bool work)
+void CoreSchematic::delete_junction(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     sheet.junctions.erase(uu);
 }
-void CoreSchematic::delete_line_net(const UUID &uu, bool work)
+void CoreSchematic::delete_line_net(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     sheet.net_lines.erase(uu);
 }
-void CoreSchematic::delete_schematic_symbol(const UUID &uu, bool work)
+void CoreSchematic::delete_schematic_symbol(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     sheet.symbols.erase(uu);
 }
-SchematicSymbol *CoreSchematic::insert_schematic_symbol(const UUID &uu, const Symbol *sym, bool work)
+SchematicSymbol *CoreSchematic::insert_schematic_symbol(const UUID &uu, const Symbol *sym)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     auto x = sheet.symbols.emplace(std::make_pair(uu, SchematicSymbol{uu, sym}));
@@ -102,31 +103,31 @@ SchematicSymbol *CoreSchematic::insert_schematic_symbol(const UUID &uu, const Sy
     return nullptr;
 }
 
-Line *CoreSchematic::insert_line(const UUID &uu, bool work)
+Line *CoreSchematic::insert_line(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     auto x = sheet.lines.emplace(std::make_pair(uu, uu));
     return &(x.first->second);
 }
-void CoreSchematic::delete_line(const UUID &uu, bool work)
+void CoreSchematic::delete_line(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     sheet.lines.erase(uu);
 }
 
-Arc *CoreSchematic::insert_arc(const UUID &uu, bool work)
+Arc *CoreSchematic::insert_arc(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     auto x = sheet.arcs.emplace(std::make_pair(uu, uu));
     return &(x.first->second);
 }
-void CoreSchematic::delete_arc(const UUID &uu, bool work)
+void CoreSchematic::delete_arc(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     sheet.arcs.erase(uu);
 }
 
-std::vector<LineNet *> CoreSchematic::get_net_lines(bool work)
+std::vector<LineNet *> CoreSchematic::get_net_lines()
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     std::vector<LineNet *> r;
@@ -135,7 +136,7 @@ std::vector<LineNet *> CoreSchematic::get_net_lines(bool work)
     }
     return r;
 }
-std::vector<NetLabel *> CoreSchematic::get_net_labels(bool work)
+std::vector<NetLabel *> CoreSchematic::get_net_labels()
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     std::vector<NetLabel *> r;
@@ -145,7 +146,7 @@ std::vector<NetLabel *> CoreSchematic::get_net_labels(bool work)
     return r;
 }
 
-std::vector<Line *> CoreSchematic::get_lines(bool work)
+std::vector<Line *> CoreSchematic::get_lines()
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     std::vector<Line *> r;
@@ -155,7 +156,7 @@ std::vector<Line *> CoreSchematic::get_lines(bool work)
     return r;
 }
 
-std::vector<Arc *> CoreSchematic::get_arcs(bool work)
+std::vector<Arc *> CoreSchematic::get_arcs()
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     std::vector<Arc *> r;
@@ -165,12 +166,12 @@ std::vector<Arc *> CoreSchematic::get_arcs(bool work)
     return r;
 }
 
-void CoreSchematic::delete_text(const UUID &uu, bool work)
+void CoreSchematic::delete_text(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     sheet.texts.erase(uu);
 }
-Text *CoreSchematic::insert_text(const UUID &uu, bool work)
+Text *CoreSchematic::insert_text(const UUID &uu)
 {
     auto &sheet = sch.sheets.at(sheet_uuid);
     auto x = sheet.texts.emplace(std::make_pair(uu, uu));
@@ -285,6 +286,13 @@ bool CoreSchematic::get_property(ObjectType type, const UUID &uu, ObjectProperty
                 dynamic_cast<PropertyValueString &>(value).value = "<no part>";
             return true;
 
+        case ObjectProperty::ID::NOPOPULATE:
+            if (block.components.at(uu).part)
+                dynamic_cast<PropertyValueBool &>(value).value = comp->nopopulate;
+            else
+                dynamic_cast<PropertyValueBool &>(value).value = false;
+            return true;
+
         default:
             return false;
         }
@@ -343,6 +351,10 @@ bool CoreSchematic::set_property(ObjectType type, const UUID &uu, ObjectProperty
             comp->value = dynamic_cast<const PropertyValueString &>(value).value;
             break;
 
+        case ObjectProperty::ID::NOPOPULATE:
+            comp->nopopulate = dynamic_cast<const PropertyValueBool &>(value).value;
+            break;
+
         default:
             return false;
         }
@@ -515,6 +527,9 @@ std::string CoreSchematic::get_display_name(ObjectType type, const UUID &uu, con
     case ObjectType::BUS_RIPPER:
         return sheet.bus_rippers.at(uu).bus_member->net->name;
 
+    case ObjectType::TEXT:
+        return sheet.texts.at(uu).text;
+
     default:
         return Core::get_display_name(type, uu);
     }
@@ -572,135 +587,6 @@ const Sheet *CoreSchematic::get_canvas_data()
     return &sch.sheets.at(sheet_uuid);
 }
 
-void CoreSchematic::commit()
-{
-    set_needs_save(true);
-}
-
-void CoreSchematic::revert()
-{
-    history_load(history_current);
-    reverted = true;
-}
-
-bool CoreSchematic::can_search_for_object_type(ObjectType ty) const
-{
-    switch (ty) {
-    case ObjectType::SCHEMATIC_SYMBOL:
-    case ObjectType::NET_LABEL:
-    case ObjectType::POWER_SYMBOL:
-    case ObjectType::BUS_RIPPER:
-        return true;
-        break;
-    default:;
-    }
-
-    return false;
-}
-
-static void sort_search_results_schematic(std::list<Core::SearchResult> &results, const Core::SearchQuery &q,
-                                          CoreSchematic *core)
-{
-    results.sort([core, q](const auto &a, const auto &b) {
-        int index_a = core->get_schematic()->sheets.at(a.sheet).index;
-        int index_b = core->get_schematic()->sheets.at(b.sheet).index;
-
-        if (a.sheet == core->get_sheet()->uuid)
-            index_a = -1;
-        if (b.sheet == core->get_sheet()->uuid)
-            index_b = -1;
-
-        if (index_a > index_b)
-            return false;
-        if (index_a < index_b)
-            return true;
-
-        auto da = core->get_display_name(a.type, a.uuid, a.sheet);
-        auto db = core->get_display_name(b.type, b.uuid, b.sheet);
-        auto ina = !Coordf(a.location).in_range(q.area_visible.first, q.area_visible.second);
-        auto inb = !Coordf(b.location).in_range(q.area_visible.first, q.area_visible.second);
-        if (ina > inb)
-            return false;
-        else if (ina < inb)
-            return true;
-
-        if (a.type > b.type)
-            return false;
-        else if (a.type < b.type)
-            return true;
-
-        auto c = strcmp_natural(da, db);
-        if (c > 0)
-            return false;
-        else if (c < 0)
-            return true;
-
-        if (a.location.x > b.location.x)
-            return false;
-        else if (a.location.x < b.location.x)
-            return true;
-
-        return a.location.y > b.location.y;
-    });
-}
-
-std::list<Core::SearchResult> CoreSchematic::search(const SearchQuery &q)
-{
-    std::list<Core::SearchResult> results;
-    if (q.get_query().size() == 0)
-        return results;
-    for (const auto &it_sheet : sch.sheets) {
-        const auto &sheet = it_sheet.second;
-        if (q.types.count(ObjectType::SCHEMATIC_SYMBOL)) {
-            for (const auto &it : sheet.symbols) {
-                if (q.contains(it.second.component->refdes)) {
-                    results.emplace_back(ObjectType::SCHEMATIC_SYMBOL, it.first);
-                    auto &x = results.back();
-                    x.location = it.second.placement.shift;
-                    x.sheet = sheet.uuid;
-                    x.selectable = true;
-                }
-            }
-        }
-        if (q.types.count(ObjectType::NET_LABEL)) {
-            for (const auto &it : sheet.net_labels) {
-                if (it.second.junction->net && q.contains(it.second.junction->net->name)) {
-                    results.emplace_back(ObjectType::NET_LABEL, it.first);
-                    auto &x = results.back();
-                    x.location = it.second.junction->position;
-                    x.sheet = sheet.uuid;
-                    x.selectable = true;
-                }
-            }
-        }
-        if (q.types.count(ObjectType::BUS_RIPPER)) {
-            for (const auto &it : sheet.bus_rippers) {
-                if (it.second.bus_member->net && q.contains(it.second.bus_member->net->name)) {
-                    results.emplace_back(ObjectType::BUS_RIPPER, it.first);
-                    auto &x = results.back();
-                    x.location = it.second.get_connector_pos();
-                    x.sheet = sheet.uuid;
-                    x.selectable = true;
-                }
-            }
-        }
-        if (q.types.count(ObjectType::POWER_SYMBOL)) {
-            for (const auto &it : sheet.power_symbols) {
-                if (it.second.junction->net && q.contains(it.second.junction->net->name)) {
-                    results.emplace_back(ObjectType::POWER_SYMBOL, it.first);
-                    auto &x = results.back();
-                    x.location = it.second.junction->position;
-                    x.sheet = sheet.uuid;
-                    x.selectable = true;
-                }
-            }
-        }
-    }
-    sort_search_results_schematic(results, q, this);
-
-    return results;
-}
-
 void CoreSchematic::history_push()
 {
     history.push_back(std::make_unique<CoreSchematic::HistoryItem>(block, sch));
