@@ -5,38 +5,37 @@
 #include <deque>
 #include <iostream>
 #include <memory>
+#include "document/idocument_symbol.hpp"
 
 namespace horizon {
-class CoreSymbol : public Core {
+class CoreSymbol : public Core, public IDocumentSymbol {
 public:
     CoreSymbol(const std::string &filename, Pool &pool);
     bool has_object_type(ObjectType ty) const override;
-    Symbol *get_symbol(bool work = true);
+    Symbol *get_symbol() override;
 
-    Junction *get_junction(const UUID &uu, bool work = true) override;
-    Line *get_line(const UUID &uu, bool work = true) override;
-    SymbolPin *get_symbol_pin(const UUID &uu, bool work = true);
-    Arc *get_arc(const UUID &uu, bool work = true) override;
+    Junction *get_junction(const UUID &uu) override;
+    Line *get_line(const UUID &uu) override;
+    SymbolPin *get_symbol_pin(const UUID &uu) override;
+    Arc *get_arc(const UUID &uu) override;
 
-    Junction *insert_junction(const UUID &uu, bool work = true) override;
-    void delete_junction(const UUID &uu, bool work = true) override;
-    Line *insert_line(const UUID &uu, bool work = true) override;
-    void delete_line(const UUID &uu, bool work = true) override;
-    Arc *insert_arc(const UUID &uu, bool work = true) override;
-    void delete_arc(const UUID &uu, bool work = true) override;
+    Junction *insert_junction(const UUID &uu) override;
+    void delete_junction(const UUID &uu) override;
+    Line *insert_line(const UUID &uu) override;
+    void delete_line(const UUID &uu) override;
+    Arc *insert_arc(const UUID &uu) override;
+    void delete_arc(const UUID &uu) override;
 
-    SymbolPin *insert_symbol_pin(const UUID &uu, bool work = true);
-    void delete_symbol_pin(const UUID &uu, bool work = true);
+    SymbolPin *insert_symbol_pin(const UUID &uu) override;
+    void delete_symbol_pin(const UUID &uu) override;
 
     class LayerProvider *get_layer_provider() override;
 
-    std::vector<Line *> get_lines(bool work = true) override;
-    std::vector<Arc *> get_arcs(bool work = true) override;
-    std::vector<const Pin *> get_pins(bool work = true);
+    std::vector<Line *> get_lines() override;
+    std::vector<Arc *> get_arcs() override;
+    std::vector<const Pin *> get_pins() override;
 
     void rebuild(bool from_undo = false) override;
-    void commit() override;
-    void revert() override;
 
     bool set_property(ObjectType type, const UUID &uu, ObjectProperty::ID property,
                       const class PropertyValue &value) override;
@@ -50,9 +49,6 @@ public:
     const Symbol *get_canvas_data();
     std::pair<Coordi, Coordi> get_bbox() override;
 
-    bool can_search_for_object_type(ObjectType type) const override;
-    std::list<SearchResult> search(const SearchQuery &q) override;
-
     void reload_pool() override;
 
     void set_pin_display_mode(Symbol::PinDisplayMode mode);
@@ -60,8 +56,8 @@ public:
     const std::string &get_filename() const override;
 
 private:
-    std::map<UUID, Text> *get_text_map(bool work = true) override;
-    std::map<UUID, Polygon> *get_polygon_map(bool work = true) override;
+    std::map<UUID, Text> *get_text_map() override;
+    std::map<UUID, Polygon> *get_polygon_map() override;
 
     Symbol sym;
     std::string m_filename;
