@@ -1,75 +1,82 @@
 #include "core.hpp"
-#include "tool_add_part.hpp"
-#include "tool_add_vertex.hpp"
-#include "tool_assign_part.hpp"
-#include "tool_bend_line_net.hpp"
-#include "tool_delete.hpp"
-#include "tool_disconnect.hpp"
-#include "tool_drag_keep_slope.hpp"
-#include "tool_draw_arc.hpp"
-#include "tool_draw_dimension.hpp"
-#include "tool_draw_line.hpp"
-#include "tool_draw_line_net.hpp"
-#include "tool_draw_line_rectangle.hpp"
-#include "tool_draw_polygon.hpp"
-#include "tool_draw_polygon_rectangle.hpp"
-#include "tool_draw_track.hpp"
-#include "tool_edit_board_hole.hpp"
-#include "tool_edit_line_rectangle.hpp"
-#include "tool_edit_pad_parameter_set.hpp"
-#include "tool_edit_parameter_program.hpp"
-#include "tool_edit_plane.hpp"
-#include "tool_edit_shape.hpp"
-#include "tool_edit_symbol_pin_names.hpp"
-#include "tool_edit_via.hpp"
-#include "tool_enter_datum.hpp"
-#include "tool_import_dxf.hpp"
-#include "tool_lock.hpp"
-#include "tool_manage_buses.hpp"
-#include "tool_map_package.hpp"
-#include "tool_map_pin.hpp"
-#include "tool_map_symbol.hpp"
-#include "tool_move.hpp"
-#include "tool_move_net_segment.hpp"
-#include "tool_paste.hpp"
-#include "tool_place_board_hole.hpp"
-#include "tool_place_bus_label.hpp"
-#include "tool_place_bus_ripper.hpp"
-#include "tool_place_hole.hpp"
-#include "tool_place_junction.hpp"
-#include "tool_place_net_label.hpp"
-#include "tool_place_pad.hpp"
-#include "tool_place_power_symbol.hpp"
-#include "tool_place_shape.hpp"
-#include "tool_place_text.hpp"
-#include "tool_place_via.hpp"
-#include "tool_rotate_arbitrary.hpp"
-#include "tool_route_track.hpp"
-#include "tool_route_track_interactive.hpp"
-#include "tool_set_diffpair.hpp"
-#include "tool_set_via_net.hpp"
-#include "tool_smash.hpp"
-#include "tool_update_all_planes.hpp"
-#include "tool_generate_courtyard.hpp"
-#include "tool_set_group.hpp"
-#include "tool_copy_placement.hpp"
-#include "tool_copy_tracks.hpp"
-#include "tool_swap_nets.hpp"
-#include "tool_line_loop_to_polygon.hpp"
-#include "tool_change_unit.hpp"
-#include "tool_set_nc_all.hpp"
-#include "tool_set_nc.hpp"
-#include "tool_add_keepout.hpp"
-#include "tool_change_symbol.hpp"
-#include "tool_place_refdes_and_value.hpp"
-#include "tool_draw_polygon_circle.hpp"
-#include "tool_draw_connection_line.hpp"
-#include "tool_backannotate_connection_lines.hpp"
-#include "tool_import_kicad_package.hpp"
-#include "tool_smash_silkscreen_graphics.hpp"
-#include "tool_renumber_pads.hpp"
-#include "tool_fix.hpp"
-#include "tool_polygon_to_line_loop.hpp"
+#include "tool_id.hpp"
+#include "tools/tool_add_part.hpp"
+#include "tools/tool_add_vertex.hpp"
+#include "tools/tool_assign_part.hpp"
+#include "tools/tool_bend_line_net.hpp"
+#include "tools/tool_delete.hpp"
+#include "tools/tool_disconnect.hpp"
+#include "tools/tool_drag_keep_slope.hpp"
+#include "tools/tool_draw_arc.hpp"
+#include "tools/tool_draw_dimension.hpp"
+#include "tools/tool_draw_line.hpp"
+#include "tools/tool_draw_line_net.hpp"
+#include "tools/tool_draw_line_rectangle.hpp"
+#include "tools/tool_draw_polygon.hpp"
+#include "tools/tool_draw_polygon_rectangle.hpp"
+#include "tools/tool_draw_track.hpp"
+#include "tools/tool_edit_board_hole.hpp"
+#include "tools/tool_edit_line_rectangle.hpp"
+#include "tools/tool_edit_pad_parameter_set.hpp"
+#include "tools/tool_edit_plane.hpp"
+#include "tools/tool_edit_shape.hpp"
+#include "tools/tool_edit_symbol_pin_names.hpp"
+#include "tools/tool_edit_via.hpp"
+#include "tools/tool_enter_datum.hpp"
+#include "tools/tool_import_dxf.hpp"
+#include "tools/tool_lock.hpp"
+#include "tools/tool_manage_buses.hpp"
+#include "tools/tool_map_package.hpp"
+#include "tools/tool_map_pin.hpp"
+#include "tools/tool_map_symbol.hpp"
+#include "tools/tool_move.hpp"
+#include "tools/tool_move_net_segment.hpp"
+#include "tools/tool_paste.hpp"
+#include "tools/tool_place_board_hole.hpp"
+#include "tools/tool_place_bus_label.hpp"
+#include "tools/tool_place_bus_ripper.hpp"
+#include "tools/tool_place_hole.hpp"
+#include "tools/tool_place_junction.hpp"
+#include "tools/tool_place_net_label.hpp"
+#include "tools/tool_place_pad.hpp"
+#include "tools/tool_place_power_symbol.hpp"
+#include "tools/tool_place_shape.hpp"
+#include "tools/tool_place_text.hpp"
+#include "tools/tool_place_via.hpp"
+#include "tools/tool_rotate_arbitrary.hpp"
+#include "tools/tool_route_track.hpp"
+#include "tools/tool_route_track_interactive.hpp"
+#include "tools/tool_set_diffpair.hpp"
+#include "tools/tool_set_via_net.hpp"
+#include "tools/tool_smash.hpp"
+#include "tools/tool_update_all_planes.hpp"
+#include "tools/tool_generate_courtyard.hpp"
+#include "tools/tool_generate_silkscreen.hpp"
+#include "tools/tool_set_group.hpp"
+#include "tools/tool_copy_placement.hpp"
+#include "tools/tool_copy_tracks.hpp"
+#include "tools/tool_swap_nets.hpp"
+#include "tools/tool_line_loop_to_polygon.hpp"
+#include "tools/tool_change_unit.hpp"
+#include "tools/tool_set_nc_all.hpp"
+#include "tools/tool_set_nc.hpp"
+#include "tools/tool_add_keepout.hpp"
+#include "tools/tool_change_symbol.hpp"
+#include "tools/tool_place_refdes_and_value.hpp"
+#include "tools/tool_draw_polygon_circle.hpp"
+#include "tools/tool_draw_connection_line.hpp"
+#include "tools/tool_backannotate_connection_lines.hpp"
+#include "tools/tool_import_kicad_package.hpp"
+#include "tools/tool_smash_silkscreen_graphics.hpp"
+#include "tools/tool_renumber_pads.hpp"
+#include "tools/tool_fix.hpp"
+#include "tools/tool_nopopulate.hpp"
+#include "tools/tool_polygon_to_line_loop.hpp"
+#include "tools/tool_place_board_panel.hpp"
+#include "tools/tool_smash_panel_outline.hpp"
+#include "tools/tool_smash_package_outline.hpp"
+#include "tools/tool_resize_symbol.hpp"
+#include "tools/tool_round_off_vertex.hpp"
 
 namespace horizon {
 
@@ -90,6 +97,8 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::MOVE_KEY_FINE_DOWN:
     case ToolID::MOVE_KEY_FINE_LEFT:
     case ToolID::MOVE_KEY_FINE_RIGHT:
+    case ToolID::ROTATE_CURSOR:
+    case ToolID::MIRROR_CURSOR:
         return std::make_unique<ToolMove>(this, tool_id);
 
     case ToolID::PLACE_JUNCTION:
@@ -114,6 +123,7 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
         return std::make_unique<ToolDrawLineNet>(this, tool_id);
 
     case ToolID::ADD_COMPONENT:
+    case ToolID::ADD_PART:
         return std::make_unique<ToolAddPart>(this, tool_id);
 
     case ToolID::PLACE_TEXT:
@@ -153,6 +163,7 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::MANAGE_POWER_NETS:
     case ToolID::EDIT_FRAME_PROPERTIES:
     case ToolID::TOGGLE_GROUP_TAG_VISIBLE:
+    case ToolID::MANAGE_INCLUDED_BOARDS:
         return std::make_unique<ToolManageBuses>(this, tool_id);
 
     case ToolID::DRAW_POLYGON:
@@ -189,12 +200,7 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::DRAG_KEEP_SLOPE:
         return std::make_unique<ToolDragKeepSlope>(this, tool_id);
 
-    case ToolID::ADD_PART:
-        return std::make_unique<ToolAddPart>(this, tool_id);
-
     case ToolID::SMASH:
-        return std::make_unique<ToolSmash>(this, tool_id);
-
     case ToolID::UNSMASH:
         return std::make_unique<ToolSmash>(this, tool_id);
 
@@ -207,12 +213,6 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::IMPORT_DXF:
         return std::make_unique<ToolImportDXF>(this, tool_id);
 
-    case ToolID::EDIT_PARAMETER_PROGRAM:
-        return std::make_unique<ToolEditParameterProgram>(this, tool_id);
-
-    case ToolID::EDIT_PARAMETER_SET:
-        return std::make_unique<ToolEditParameterProgram>(this, tool_id);
-
     case ToolID::EDIT_PAD_PARAMETER_SET:
         return std::make_unique<ToolEditPadParameterSet>(this, tool_id);
 
@@ -241,8 +241,6 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
         return std::make_unique<ToolRotateArbitrary>(this, tool_id);
 
     case ToolID::ADD_PLANE:
-        return std::make_unique<ToolEditPlane>(this, tool_id);
-
     case ToolID::EDIT_PLANE:
     case ToolID::CLEAR_PLANE:
     case ToolID::UPDATE_PLANE:
@@ -280,6 +278,9 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::GENERATE_COURTYARD:
         return std::make_unique<ToolGenerateCourtyard>(this, tool_id);
 
+    case ToolID::GENERATE_SILKSCREEN:
+        return std::make_unique<ToolGenerateSilkscreen>(this, tool_id);
+
     case ToolID::SET_GROUP:
     case ToolID::SET_NEW_GROUP:
     case ToolID::CLEAR_GROUP:
@@ -345,9 +346,28 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::UNFIX:
         return std::make_unique<ToolFix>(this, tool_id);
 
+    case ToolID::NOPOPULATE:
+    case ToolID::POPULATE:
+        return std::make_unique<ToolNoPopulate>(this, tool_id);
+
     case ToolID::POLYGON_TO_LINE_LOOP:
         return std::make_unique<ToolPolygonToLineLoop>(this, tool_id);
 
+    case ToolID::PLACE_BOARD_PANEL:
+        return std::make_unique<ToolPlaceBoardPanel>(this, tool_id);
+
+    case ToolID::SMASH_PANEL_OUTLINE:
+        return std::make_unique<ToolSmashPanelOutline>(this, tool_id);
+
+    case ToolID::SMASH_PACKAGE_OUTLINE:
+        return std::make_unique<ToolSmashPackageOutline>(this, tool_id);
+
+    case ToolID::RESIZE_SYMBOL:
+        return std::make_unique<ToolResizeSymbol>(this, tool_id);
+
+    case ToolID::ROUND_OFF_VERTEX:
+        return std::make_unique<ToolRoundOffVertex>(this, tool_id);
+
     default:
         return nullptr;
     }
