@@ -210,10 +210,12 @@ View3DWindow::View3DWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Buil
         }
     });
 
-    Gtk::Revealer *model_loading_revealer;
     x->get_widget("model_loading_revealer", model_loading_revealer);
-    canvas->signal_models_loading().connect(
-            [model_loading_revealer](bool v) { model_loading_revealer->set_reveal_child(v); });
+    x->get_widget("model_loading_spinner", model_loading_spinner);
+    canvas->signal_models_loading().connect([this](bool v) {
+        model_loading_revealer->set_reveal_child(v);
+        model_loading_spinner->property_active() = v;
+    });
 
     Gtk::ComboBoxText *msaa_combo;
     x->get_widget("msaa_combo", msaa_combo);
@@ -243,8 +245,7 @@ void View3DWindow::set_smooth_zoom(bool smooth)
 void View3DWindow::update(bool clear)
 {
     s_signal_request_update.emit();
-    canvas->patches.clear();
-    canvas->update2(*board);
+    canvas->update(*board);
     if (clear)
         canvas->clear_3d_models();
     canvas->load_models_async(pool);
