@@ -1,5 +1,6 @@
 #include "action_catalog.hpp"
 #include "core/tool_id.hpp"
+#include "imp/action.hpp"
 
 namespace horizon {
 const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog = {
@@ -542,6 +543,10 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
          {"Generate courtyard", ActionGroup::PACKAGE, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::TOOL, ToolID::GENERATE_SILKSCREEN},
+         {"Generate silkscreen", ActionGroup::PACKAGE, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::TOOL, ToolID::MOVE_KEY},
          {"Move by keyboard", ActionGroup::MOVE, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_DEFAULT}},
@@ -714,6 +719,14 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
          {"Unfix package", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::TOOL, ToolID::NOPOPULATE},
+         {"Mark do not populate", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::POPULATE},
+         {"Mark populate", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::TOOL, ToolID::POLYGON_TO_LINE_LOOP},
          {"Polygon to line loop", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_DEFAULT}},
@@ -724,6 +737,62 @@ const std::map<std::pair<ActionID, ToolID>, ActionCatalogItem> action_catalog =
 
         {{ActionID::EXPORT_STEP, ToolID::NONE},
          {"Export STEP", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::PNP_EXPORT_WINDOW, ToolID::NONE},
+         {"Pick & place export window", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::EXPORT_PNP, ToolID::NONE},
+         {"Export Pick & place", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::AIRWIRE_FILTER_WINDOW, ToolID::NONE},
+         {"Airwire filter", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::RESET_AIRWIRE_FILTER, ToolID::NONE},
+         {"Reset airwire filter", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::FILTER_AIRWIRES, ToolID::NONE},
+         {"Filter airwires", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
+        {{ActionID::SELECT_POLYGON, ToolID::NONE},
+         {"Select polygon", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
+        {{ActionID::TOOL, ToolID::ROTATE_CURSOR},
+         {"Rotate around cursor", ActionGroup::MOVE, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::MIRROR_CURSOR},
+         {"Mirror around cursor", ActionGroup::MOVE, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::PLACE_BOARD_PANEL},
+         {"Place board panel", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::MANAGE_INCLUDED_BOARDS},
+         {"Manage included boards", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::SMASH_PANEL_OUTLINE},
+         {"Smash panel outline", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::SMASH_PACKAGE_OUTLINE},
+         {"Smash package outline", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::RESIZE_SYMBOL},
+         {"Resize symbol", ActionGroup::SYMBOL, ActionCatalogItem::AVAILABLE_IN_SYMBOL,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::ROUND_OFF_VERTEX},
+         {"Round off vertex", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
 };
 
 const std::vector<std::pair<ActionGroup, std::string>> action_group_catalog = {
@@ -814,6 +883,12 @@ const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(SELECT_MORE_NO_VIA),
         ACTION_LUT_ITEM(EXPORT_STEP),
         ACTION_LUT_ITEM(STEP_EXPORT_WINDOW),
+        ACTION_LUT_ITEM(EXPORT_PNP),
+        ACTION_LUT_ITEM(PNP_EXPORT_WINDOW),
+        ACTION_LUT_ITEM(RESET_AIRWIRE_FILTER),
+        ACTION_LUT_ITEM(FILTER_AIRWIRES),
+        ACTION_LUT_ITEM(AIRWIRE_FILTER_WINDOW),
+        ACTION_LUT_ITEM(SELECT_POLYGON),
 };
 
 #define TOOL_LUT_ITEM(x)                                                                                               \
@@ -869,8 +944,6 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(EDIT_SHAPE),
         TOOL_LUT_ITEM(IMPORT_DXF),
         TOOL_LUT_ITEM(MANAGE_NET_CLASSES),
-        TOOL_LUT_ITEM(EDIT_PARAMETER_SET),
-        TOOL_LUT_ITEM(EDIT_PARAMETER_PROGRAM),
         TOOL_LUT_ITEM(EDIT_PAD_PARAMETER_SET),
         TOOL_LUT_ITEM(DRAW_POLYGON_RECTANGLE),
         TOOL_LUT_ITEM(DRAW_LINE_RECTANGLE),
@@ -900,6 +973,7 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(PLACE_BOARD_HOLE),
         TOOL_LUT_ITEM(EDIT_BOARD_HOLE),
         TOOL_LUT_ITEM(GENERATE_COURTYARD),
+        TOOL_LUT_ITEM(GENERATE_SILKSCREEN),
         TOOL_LUT_ITEM(SET_GROUP),
         TOOL_LUT_ITEM(SET_NEW_GROUP),
         TOOL_LUT_ITEM(CLEAR_GROUP),
@@ -945,6 +1019,17 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(RENUMBER_PADS),
         TOOL_LUT_ITEM(FIX),
         TOOL_LUT_ITEM(UNFIX),
+        TOOL_LUT_ITEM(NOPOPULATE),
+        TOOL_LUT_ITEM(POPULATE),
         TOOL_LUT_ITEM(POLYGON_TO_LINE_LOOP),
+        TOOL_LUT_ITEM(ROTATE_CURSOR),
+        TOOL_LUT_ITEM(MIRROR_CURSOR),
+        TOOL_LUT_ITEM(PLACE_BOARD_PANEL),
+        TOOL_LUT_ITEM(MANAGE_INCLUDED_BOARDS),
+        TOOL_LUT_ITEM(RELOAD_INCLUDED_BOARDS),
+        TOOL_LUT_ITEM(SMASH_PANEL_OUTLINE),
+        TOOL_LUT_ITEM(SMASH_PACKAGE_OUTLINE),
+        TOOL_LUT_ITEM(RESIZE_SYMBOL),
+        TOOL_LUT_ITEM(ROUND_OFF_VERTEX),
 };
 } // namespace horizon
