@@ -9,22 +9,27 @@
 #include "widgets/board_display_options.hpp"
 #include "widgets/layer_box.hpp"
 #include "tuning_window.hpp"
-#include "hud_util.hpp"
+#include "util/selection_util.hpp"
 #include "util/util.hpp"
 #include "util/str_util.hpp"
 #include "canvas/annotation.hpp"
 #include "export_pdf/export_pdf_board.hpp"
 #include "board/board_layers.hpp"
 #include "pdf_export_window.hpp"
+#include "widgets/unplaced_box.hpp"
+#include "pnp_export_window.hpp"
+#include "airwire_filter_window.hpp"
+#include "core/tool_id.hpp"
 
 namespace horizon {
 ImpBoard::ImpBoard(const std::string &board_filename, const std::string &block_filename, const std::string &via_dir,
                    const PoolParams &pool_params)
     : ImpLayer(pool_params), core_board(board_filename, block_filename, via_dir, *pool),
-      project_dir(Glib::path_get_dirname(board_filename))
+      project_dir(Glib::path_get_dirname(board_filename)), searcher(core_board), airwire_filter(*core_board.get_board())
 {
     core = &core_board;
     core_board.signal_tool_changed().connect(sigc::mem_fun(*this, &ImpBase::handle_tool_change));
+    load_meta();
 }
 
 void ImpBoard::canvas_update()
@@ -48,9 +53,10 @@ void ImpBoard::update_highlights()
                     canvas->set_flags(ref, Triangle::FLAG_HIGHLIGHT, 0);
                 }
             }
-            for (const auto &it_track : core_board.get_board()->airwires) {
-                if (it_track.second.net.uuid == it.uuid) {
-                    ObjectRef ref(ObjectType::TRACK, it_track.first);
+            {
+                const auto &airwires = core_board.get_board()->airwires;
+                if (airwires.count(it.uuid)) {
+                    ObjectRef ref(ObjectType::AIRWIRE, it.uuid);
                     canvas->set_flags(ref, Triangle::FLAG_HIGHLIGHT, 0);
                 }
             }
@@ -209,6 +215,7 @@ void ImpBoard::update_action_sensitivity()
     set_action_sensitive(make_action(ActionID::HIGHLIGHT_NET), can_select_more);
     set_action_sensitive(make_action(ActionID::SELECT_MORE), can_select_more);
     set_action_sensitive(make_action(ActionID::SELECT_MORE_NO_VIA), can_select_more);
+    set_action_sensitive(make_action(ActionID::FILTER_AIRWIRES), can_select_more || n_pkgs);
 
     set_action_sensitive(make_action(ActionID::GO_TO_SCHEMATIC), sockets_connected);
     set_action_sensitive(make_action(ActionID::SHOW_IN_POOL_MANAGER), n_pkgs == 1 && sockets_connected);
@@ -339,7 +346,6 @@ void ImpBoard::construct()
 {
     ImpLayer::construct_layer_box(false);
 
-    main_window->set_title("Board - Interactive Manipulator");
     state_store = std::make_unique<WindowStateStore>(main_window, "imp-board");
 
     auto view_3d_button = Gtk::manage(new Gtk::Button("3D"));
@@ -371,6 +377,9 @@ void ImpBoard::construct()
     hamburger_menu->append("Export STEP", "win.export_step");
     main_window->add_action("export_step", [this] { trigger_action(ActionID::STEP_EXPORT_WINDOW); });
 
+    hamburger_menu->append("Export Pick & place", "win.export_pnp");
+    main_window->add_action("export_pnp", [this] { trigger_action(ActionID::PNP_EXPORT_WINDOW); });
+
     hamburger_menu->append("Length tuning", "win.tuning");
     main_window->add_action("tuning", [this] { trigger_action(ActionID::TUNING); });
 
@@ -390,9 +399,8 @@ void ImpBoard::construct()
         canvas->signal_selection_changed().connect(sigc::mem_fun(*this, &ImpBoard::handle_selection_cross_probe));
 
         connect_action(ActionID::GO_TO_SCHEMATIC, [this](const auto &conn) {
-            auto ev = gtk_get_current_event();
             json j;
-            j["time"] = gdk_event_get_time(ev);
+            j["time"] = gtk_get_current_event_time();
             j["op"] = "present-schematic";
             auto sch_pid = this->get_schematic_pid();
             if (sch_pid != -1)
@@ -422,9 +430,8 @@ void ImpBoard::construct()
         set_action_sensitive(make_action(ActionID::SHOW_IN_POOL_MANAGER), false);
 
         connect_action(ActionID::BACKANNOTATE_CONNECTION_LINES, [this](const auto &conn) {
-            auto ev = gtk_get_current_event();
             json j;
-            j["time"] = gdk_event_get_time(ev);
+            j["time"] = gtk_get_current_event_time();
             j["op"] = "backannotate";
             allow_set_foreground_window(this->get_schematic_pid());
             json a;
@@ -456,14 +463,14 @@ void ImpBoard::construct()
     }
 
     fab_output_window = FabOutputWindow::create(main_window, &core_board, project_dir);
-    core.r->signal_tool_changed().connect([this](ToolID t) { fab_output_window->set_can_generate(t == ToolID::NONE); });
-    core.r->signal_rebuilt().connect([this] { fab_output_window->reload_layers(); });
+    core->signal_tool_changed().connect([this](ToolID t) { fab_output_window->set_can_generate(t == ToolID::NONE); });
+    core->signal_rebuilt().connect([this] { fab_output_window->reload_layers(); });
     fab_output_window->signal_changed().connect([this] { core_board.set_needs_save(); });
 
     pdf_export_window =
             PDFExportWindow::create(main_window, &core_board, *core_board.get_pdf_export_settings(), project_dir);
     pdf_export_window->signal_changed().connect([this] { core_board.set_needs_save(); });
-    core.r->signal_rebuilt().connect([this] { pdf_export_window->reload_layers(); });
+    core->signal_rebuilt().connect([this] { pdf_export_window->reload_layers(); });
     connect_action(ActionID::PDF_EXPORT_WINDOW, [this](const auto &c) { pdf_export_window->present(); });
     connect_action(ActionID::EXPORT_PDF, [this](const auto &c) {
         pdf_export_window->present();
@@ -487,7 +494,7 @@ void ImpBoard::construct()
         step_export_window->generate();
     });
 
-    tuning_window = new TuningWindow(core.b->get_board());
+    tuning_window = new TuningWindow(core_board.get_board());
     tuning_window->set_transient_for(*main_window);
 
     rules_window->signal_goto().connect([this](Coordi location, UUID sheet) { canvas->center_and_zoom(location); });
@@ -537,17 +544,17 @@ void ImpBoard::construct()
     layer_box->show();
     layer_box->get_style_context()->add_class("background");
 
-    auto board_display_options = Gtk::manage(new BoardDisplayOptionsBox(core.b->get_layer_provider()));
+    board_display_options_box = Gtk::manage(new BoardDisplayOptionsBox(core_board.get_layer_provider()));
     {
         auto fbox = Gtk::manage(new Gtk::Box());
-        fbox->pack_start(*board_display_options, true, true, 0);
+        fbox->pack_start(*board_display_options_box, true, true, 0);
         fbox->get_style_context()->add_class("background");
         fbox->show();
         display_control_notebook->append_page(*fbox, "Options");
-        board_display_options->show();
+        board_display_options_box->show();
     }
 
-    board_display_options->signal_set_layer_display().connect([this](int index, const LayerDisplay &lda) {
+    board_display_options_box->signal_set_layer_display().connect([this](int index, const LayerDisplay &lda) {
         LayerDisplay ld = canvas->get_layer_display(index);
         ld.types_force_outline = lda.types_force_outline;
         ld.types_visible = lda.types_visible;
@@ -555,7 +562,9 @@ void ImpBoard::construct()
         canvas->queue_draw();
     });
     canvas->set_layer_display(10000, LayerDisplay(true, LayerDisplay::Mode::OUTLINE));
-    core.r->signal_rebuilt().connect([board_display_options] { board_display_options->update(); });
+    core->signal_rebuilt().connect([this] { board_display_options_box->update(); });
+    if (m_meta.count("board_display_options"))
+        board_display_options_box->load_from_json(m_meta.at("board_display_options"));
 
     canvas->signal_motion_notify_event().connect([this](GdkEventMotion *ev) {
         if (target_drag_begin.type != ObjectType::INVALID) {
@@ -581,9 +590,93 @@ void ImpBoard::construct()
     core_board.signal_rebuilt().connect([this] { selection_filter_dialog->update_layers(); });
 
     main_window->left_panel->pack_start(*display_control_notebook, false, false);
+
+    unplaced_box = Gtk::manage(new UnplacedBox("Package"));
+    unplaced_box->show();
+    main_window->left_panel->pack_end(*unplaced_box, true, true, 0);
+    unplaced_box->signal_place().connect([this](const auto &items) {
+        std::set<SelectableRef> components;
+        for (const auto &it : items) {
+            components.emplace(it.at(0), ObjectType::COMPONENT);
+        }
+        this->tool_begin(ToolID::MAP_PACKAGE, true, components);
+    });
+    core_board.signal_rebuilt().connect(sigc::mem_fun(*this, &ImpBoard::update_unplaced));
+    update_unplaced();
+    core_board.signal_tool_changed().connect(
+            [this](ToolID tool_id) { unplaced_box->set_sensitive(tool_id == ToolID::NONE); });
+
+
+    pnp_export_window = PnPExportWindow::create(main_window, *core_board.get_board(),
+                                                *core_board.get_pnp_export_settings(), project_dir);
+
+    connect_action(ActionID::PNP_EXPORT_WINDOW, [this](const auto &c) { pnp_export_window->present(); });
+    connect_action(ActionID::EXPORT_PNP, [this](const auto &c) {
+        pnp_export_window->present();
+        pnp_export_window->generate();
+    });
+
+    pnp_export_window->signal_changed().connect([this] { core_board.set_needs_save(); });
+    core->signal_tool_changed().connect([this](ToolID t) { pnp_export_window->set_can_export(t == ToolID::NONE); });
+    core->signal_rebuilt().connect([this] { pnp_export_window->update(); });
+
+    airwire_filter_window = AirwireFilterWindow::create(main_window, *core_board.get_block(), airwire_filter);
+    connect_action(ActionID::AIRWIRE_FILTER_WINDOW, [this](const auto &a) { airwire_filter_window->present(); });
+    core_board.signal_rebuilt().connect(sigc::mem_fun(*this, &ImpBoard::update_airwires));
+    update_airwires();
+    canvas->set_airwire_filter(&airwire_filter);
+    airwire_filter.signal_changed().connect([this] {
+        canvas_update_from_pp();
+        update_view_hints();
+    });
+    connect_action(ActionID::RESET_AIRWIRE_FILTER, [this](const auto &a) { airwire_filter.set_all(true); });
+    connect_action(ActionID::FILTER_AIRWIRES, [this](const auto &a) {
+        std::set<UUID> nets;
+        const auto board = core_board.get_board();
+        for (const auto &it : canvas->get_selection()) {
+            switch (it.type) {
+            case ObjectType::TRACK: {
+                auto &track = board->tracks.at(it.uuid);
+                if (track.net) {
+                    nets.emplace(track.net->uuid);
+                }
+            } break;
+            case ObjectType::VIA: {
+                auto &via = board->vias.at(it.uuid);
+                if (via.junction->net) {
+                    nets.emplace(via.junction->net->uuid);
+                }
+            } break;
+            case ObjectType::JUNCTION: {
+                auto &ju = board->junctions.at(it.uuid);
+                if (ju.net) {
+                    nets.emplace(ju.net->uuid);
+                }
+            } break;
+            case ObjectType::BOARD_PACKAGE: {
+                auto &pkg = board->packages.at(it.uuid);
+                for (const auto &it_pad : pkg.package.pads) {
+                    if (it_pad.second.net) {
+                        nets.emplace(it_pad.second.net->uuid);
+                    }
+                }
+            } break;
+            default:;
+            }
+        }
+
+        airwire_filter.set_only(nets);
+    });
+
     display_control_notebook->show();
 }
 
+void ImpBoard::update_airwires()
+{
+    airwire_filter.update_from_board();
+    airwire_filter_window->update_from_filter();
+}
+
 void ImpBoard::update_text_owners()
 {
     auto brd = core_board.get_board();
@@ -646,7 +739,7 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
         }
         s += "Layers: ";
         for (auto layer : layers) {
-            s += core.r->get_layer_provider()->get_layers().at(layer).name + " ";
+            s += core->get_layer_provider()->get_layers().at(layer).name + " ";
         }
         s += "\nTotal length: " + dim_to_string(length, false);
         if (sel_count_type(sel, ObjectType::TRACK) == 1) {
@@ -660,7 +753,7 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
     }
     trim(s);
     if (sel_count_type(sel, ObjectType::BOARD_PACKAGE) == 1) {
-        const auto &pkg = core_board.get_board()->packages.at(sel_find_one(sel, ObjectType::BOARD_PACKAGE));
+        const auto &pkg = core_board.get_board()->packages.at(sel_find_one(sel, ObjectType::BOARD_PACKAGE).uuid);
         s += "\n\n<b>Package " + pkg.component->refdes + "</b>";
         if (pkg.fixed) {
             s += " (not movable)";
@@ -746,7 +839,7 @@ void ImpBoard::handle_drag()
                 update_highlights();
                 ToolArgs args;
                 args.coords = target_drag_begin.p;
-                ToolResponse r = core.r->tool_begin(ToolID::ROUTE_TRACK_INTERACTIVE, args, imp_interface.get());
+                ToolResponse r = core->tool_begin(ToolID::ROUTE_TRACK_INTERACTIVE, args, imp_interface.get());
                 tool_process(r);
             }
             {
@@ -756,7 +849,7 @@ void ImpBoard::handle_drag()
                 args.button = 1;
                 args.target = target_drag_begin;
                 args.work_layer = canvas->property_work_layer();
-                ToolResponse r = core.r->tool_update(args);
+                ToolResponse r = core->tool_update(args);
                 tool_process(r);
             }
         }
@@ -766,7 +859,7 @@ void ImpBoard::handle_drag()
                 update_highlights();
                 ToolArgs args;
                 args.coords = target_drag_begin.p;
-                ToolResponse r = core.r->tool_begin(ToolID::DRAW_CONNECTION_LINE, args, imp_interface.get());
+                ToolResponse r = core->tool_begin(ToolID::DRAW_CONNECTION_LINE, args, imp_interface.get());
                 tool_process(r);
             }
             {
@@ -776,7 +869,7 @@ void ImpBoard::handle_drag()
                 args.button = 1;
                 args.target = target_drag_begin;
                 args.work_layer = canvas->property_work_layer();
-                ToolResponse r = core.r->tool_update(args);
+                ToolResponse r = core->tool_update(args);
                 tool_process(r);
             }
         }
@@ -830,7 +923,7 @@ static void append_bottom_layers(std::vector<int> &layers)
     std::vector<int> bottoms;
     bottoms.reserve(layers.size());
     for (auto it = layers.rbegin(); it != layers.rend(); it++) {
-        if (*it >= 0 && *it != BoardLayers::L_OUTLINE)
+        if (*it >= 0 && *it != BoardLayers::L_OUTLINE && *it != BoardLayers::OUTLINE_NOTES)
             bottoms.push_back(-100 - *it);
     }
     layers.insert(layers.end(), bottoms.begin(), bottoms.end());
@@ -842,8 +935,8 @@ std::map<ObjectType, ImpBase::SelectionFilterInfo> ImpBoard::get_selection_filte
     for (unsigned i = 0; i < core_board.get_board()->get_n_inner_layers(); i++) {
         inner_layers.push_back(-i - 1);
     }
-    std::vector<int> layers_line = {BoardLayers::TOP_ASSEMBLY, BoardLayers::TOP_SILKSCREEN, BoardLayers::TOP_MASK,
-                                    BoardLayers::TOP_COPPER};
+    std::vector<int> layers_line = {BoardLayers::OUTLINE_NOTES, BoardLayers::TOP_ASSEMBLY, BoardLayers::TOP_SILKSCREEN,
+                                    BoardLayers::TOP_MASK, BoardLayers::TOP_COPPER};
     append_bottom_layers(layers_line);
 
     std::vector<int> layers_polygon = {BoardLayers::L_OUTLINE, BoardLayers::TOP_SILKSCREEN, BoardLayers::TOP_MASK,
@@ -869,8 +962,42 @@ std::map<ObjectType, ImpBase::SelectionFilterInfo> ImpBoard::get_selection_filte
             {ObjectType::DIMENSION, {}},
             {ObjectType::BOARD_HOLE, {}},
             {ObjectType::CONNECTION_LINE, {}},
+            {ObjectType::BOARD_PANEL, {}},
     };
     return r;
 }
 
+void ImpBoard::update_unplaced()
+{
+    std::map<UUIDPath<2>, std::string> components;
+    const auto brd = core_board.get_board();
+    for (const auto &it : brd->block->components) {
+        if (it.second.part)
+            components.emplace(std::piecewise_construct, std::forward_as_tuple(it.second.uuid),
+                               std::forward_as_tuple(it.second.refdes));
+    }
+
+    for (auto &it : brd->packages) {
+        components.erase(it.second.component->uuid);
+    }
+    unplaced_box->update(components);
+}
+
+void ImpBoard::get_save_meta(json &j)
+{
+    ImpLayer::get_save_meta(j);
+    j["board_display_options"] = board_display_options_box->serialize();
+}
+
+std::vector<std::string> ImpBoard::get_view_hints()
+{
+    auto r = ImpBase::get_view_hints();
+
+    const auto &aws = airwire_filter.get_airwires();
+    if (std::any_of(aws.begin(), aws.end(), [](const auto &x) { return x.second.visible == false; }))
+        r.emplace_back("airwires filtered");
+
+    return r;
+}
+
 } // namespace horizon
