@@ -23,6 +23,9 @@ private:
     void canvas_update() override;
     CoreFrame core_frame;
 
+    class HeaderButton *header_button = nullptr;
     Gtk::Entry *name_entry = nullptr;
+
+    void update_header();
 };
 } // namespace horizon
