@@ -3,6 +3,7 @@
 #include "block/block.hpp"
 #include "board/board.hpp"
 #include "imp_layer.hpp"
+#include "search/searcher_package.hpp"
 
 namespace horizon {
 class ImpPackage : public ImpLayer {
@@ -32,9 +33,15 @@ protected:
     void update_monitor() override;
     std::pair<ActionID, ToolID> get_doubleclick_action(ObjectType type, const UUID &uu) override;
 
+    Searcher *get_searcher_ptr() override
+    {
+        return &searcher;
+    }
+
 private:
     void canvas_update() override;
     CorePackage core_package;
+    SearcherPackage searcher;
 
     Block fake_block;
     Board fake_board;
@@ -42,9 +49,15 @@ private:
     class FootprintGeneratorWindow *footprint_generator_window;
     class View3DWindow *view_3d_window = nullptr;
     std::string ask_3d_model_filename(const std::string &current_filename = "");
+    void construct_3d();
 
     Gtk::ListBox *models_listbox = nullptr;
     class LayerHelpBox *layer_help_box = nullptr;
     UUID current_model;
+
+    class HeaderButton *header_button = nullptr;
+    Gtk::Entry *entry_name = nullptr;
+
+    void update_header();
 };
 } // namespace horizon
