@@ -23,7 +23,8 @@
 #include "output_window.hpp"
 #include "util/str_util.hpp"
 #include "autosave_recovery_dialog.hpp"
-
+#include "util/item_set.hpp"
+#include "prj-mgr/pool_cache_monitor.hpp"
 
 namespace horizon {
 PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject,
@@ -129,12 +130,17 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
     download_status_dispatcher.attach(download_revealer);
     download_status_dispatcher.attach(download_label);
     download_status_dispatcher.attach(download_spinner);
+    label_set_tnum(download_label);
 
     download_status_dispatcher.signal_notified().connect([this](const StatusDispatcher::Notification &n) {
         auto is_busy = n.status == StatusDispatcher::Status::BUSY;
-        button_cancel->set_sensitive(!is_busy);
         button_do_download->set_sensitive(!is_busy);
+        downloading = is_busy;
         if (n.status == StatusDispatcher::Status::DONE) {
+            if (download_cancel) {
+                set_view_mode(ViewMode::OPEN);
+                return;
+            }
             PoolManager::get().add_pool(download_dest_dir_entry->get_text());
             if (download_back_to_start) {
                 app->recent_items[Glib::build_filename(download_dest_dir_entry->get_text(), "pool.json")] =
@@ -339,6 +345,7 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
             auto pid = proc->proc->get_pid();
             json tx;
             tx["op"] = "reload-netlist";
+            tx["time"] = timestamp;
             app->send_json(pid, tx);
         }
     }
@@ -371,6 +378,17 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
         bool needs_save = j.at("needs_save");
         std::cout << "needs save " << pid << " " << needs_save << std::endl;
         pids_need_save[pid] = needs_save;
+        if (project && !needs_save)
+            view_project.update_meta();
+    }
+    else if (op == "ready") {
+        int pid = j.at("pid");
+        for (auto &it : processes) {
+            if (it.second.proc && it.second.proc->get_pid() == pid) {
+                it.second.signal_ready().emit();
+                break;
+            }
+        }
     }
     else if (op == "edit") {
         auto type = object_type_lut.lookup(j.at("type"));
@@ -485,12 +503,22 @@ void PoolProjectManagerAppWindow::set_pool_update_progress(float progress)
 
 PoolProjectManagerAppWindow::~PoolProjectManagerAppWindow()
 {
-    if (part_browser_window)
-        delete part_browser_window;
-    if (pool_cache_window)
-        delete pool_cache_window;
     if (output_window)
         delete output_window;
+    cleanup();
+}
+
+void PoolProjectManagerAppWindow::cleanup()
+{
+    if (part_browser_window) {
+        delete part_browser_window;
+        part_browser_window = nullptr;
+    }
+    if (pool_cache_window) {
+        delete pool_cache_window;
+        pool_cache_window = nullptr;
+    }
+    pool_cache_monitor.reset();
 }
 
 void PoolProjectManagerAppWindow::handle_recent()
@@ -507,12 +535,16 @@ void PoolProjectManagerAppWindow::handle_download(bool back_to_start)
 
 void PoolProjectManagerAppWindow::handle_cancel()
 {
-    set_view_mode(ViewMode::OPEN);
+    if (downloading) {
+        download_cancel = true;
+    }
+    else {
+        set_view_mode(ViewMode::OPEN);
+    }
 }
 
 void PoolProjectManagerAppWindow::save_project()
 {
-    project->title = view_project.entry_project_title->get_text();
     save_json_to_file(project_filename, project->serialize());
     project_needs_save = false;
 }
@@ -527,7 +559,7 @@ void PoolProjectManagerAppWindow::handle_save()
 
 void PoolProjectManagerAppWindow::handle_open()
 {
-    GtkFileChooserNative *native = gtk_file_chooser_native_new("Open Pool", GTK_WINDOW(gobj()),
+    GtkFileChooserNative *native = gtk_file_chooser_native_new("Open Pool or Project", GTK_WINDOW(gobj()),
                                                                GTK_FILE_CHOOSER_ACTION_OPEN, "_Open", "_Cancel");
     auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
     auto filter = Gtk::FileFilter::create();
@@ -558,6 +590,7 @@ void PoolProjectManagerAppWindow::handle_new_project()
 {
     if (!check_pools())
         return;
+    view_create_project.clear();
     set_view_mode(ViewMode::CREATE_PROJECT);
 }
 
@@ -585,6 +618,30 @@ void PoolProjectManagerAppWindow::handle_create()
     }
 }
 
+static std::string peek_name(const std::string &path)
+{
+    std::string name;
+    try {
+        auto j = load_json_from_file(path);
+        if (Glib::path_get_basename(path) == "pool.json") {
+            name = j.at("name");
+        }
+        else {
+            auto prj = Project::new_from_file(path);
+            auto block_filename = prj.get_top_block().block_filename;
+            auto meta = Block::peek_project_meta(block_filename);
+            if (meta.count("project_title"))
+                name = meta.at("project_title");
+            if (!name.size())
+                name = j.at("title");
+        }
+    }
+    catch (...) {
+        name = "error opening!";
+    }
+    return name;
+}
+
 void PoolProjectManagerAppWindow::update_recent_items()
 {
     if (!app)
@@ -600,28 +657,17 @@ void PoolProjectManagerAppWindow::update_recent_items()
     std::vector<std::pair<std::string, Glib::DateTime>> recent_items_sorted = recent_sort(app->recent_items);
     for (const auto &it : recent_items_sorted) {
         const std::string &path = it.first;
-        std::string name;
-        try {
-            auto ifs = make_ifstream(path);
-            if (ifs.is_open()) {
-                json k;
-                ifs >> k;
-                if (endswith(path, "pool.json"))
-                    name = k.at("name");
-                else
-                    name = k.at("title");
-            }
-            ifs.close();
-        }
-        catch (...) {
-            name = "error opening!";
-        }
+        std::string name = peek_name(path);
         auto box = Gtk::manage(new RecentItemBox(name, it.first, it.second));
         if (endswith(path, "pool.json"))
             recent_pools_listbox->append(*box);
         else
             recent_projects_listbox->append(*box);
         box->show();
+        box->signal_remove().connect([this, box] {
+            app->recent_items.erase(box->path);
+            Glib::signal_idle().connect_once([this] { update_recent_items(); });
+        });
     }
 }
 
@@ -729,12 +775,7 @@ bool PoolProjectManagerAppWindow::really_close_pool_or_project()
         }
         app->recent_items[project_filename] = Glib::DateTime::create_now_local();
         project.reset();
-        if (part_browser_window)
-            delete part_browser_window;
-        part_browser_window = nullptr;
-        if (pool_cache_window)
-            delete pool_cache_window;
-        pool_cache_window = nullptr;
+        cleanup();
         set_view_mode(ViewMode::OPEN);
     }
     else {
@@ -842,9 +883,9 @@ void PoolProjectManagerAppWindow::pool_notebook_go_to(ObjectType type, const UUI
 PoolProjectManagerAppWindow *PoolProjectManagerAppWindow::create(PoolProjectManagerApplication *app)
 {
     // Load the Builder file and instantiate its widgets.
-    std::vector<Glib::ustring> widgets = {"app_window",        "sg_dest",         "sg_repo",          "menu1",
-                                          "sg_base_path",      "sg_project_name", "sg_project_title", "sg_pool_name",
-                                          "sg_pool_base_path", "sg_prj_pool"};
+    std::vector<Glib::ustring> widgets = {"app_window",        "sg_dest",         "sg_repo",           "menu1",
+                                          "sg_base_path",      "sg_project_name", "sg_project_title",  "sg_pool_name",
+                                          "sg_pool_base_path", "sg_prj_pool",     "sg_prj_open_change"};
     auto refBuilder = Gtk::Builder::create_from_resource("/org/horizon-eda/horizon/pool-prj-mgr/window.ui", widgets);
 
     PoolProjectManagerAppWindow *window = nullptr;
@@ -1013,15 +1054,12 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
         std::string pool_path = prj_pool->base_path;
         check_schema_update(pool_path);
 
-        header->set_subtitle(project->title);
         view_project.label_project_directory->set_text(Glib::path_get_dirname(project_filename));
-        view_project.entry_project_title->set_text(project->title);
-        view_project.entry_project_title->grab_focus_without_selecting();
-
 
         view_project.pool_info_bar->hide();
         view_project.label_pool_name->set_text(prj_pool->name);
         view_project.label_pool_path->set_text(prj_pool->base_path);
+        view_project.reset_pool_cache_status();
 
         part_browser_window = PartBrowserWindow::create(this, prj_pool->base_path, app->part_favorites);
         part_browser_window->signal_place_part().connect(
@@ -1029,8 +1067,37 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
         part_browser_window->signal_assign_part().connect(
                 sigc::mem_fun(*this, &PoolProjectManagerAppWindow::handle_assign_part));
         pool_cache_window = PoolCacheWindow::create(this, project->pool_cache_directory, prj_pool->base_path, this);
+        pool_cache_monitor = std::make_unique<PoolCacheMonitor>(prj_pool->base_path, project->pool_cache_directory);
+        app->signal_pool_updated().connect(sigc::track_obj(
+                [this](const UUID &pool_uu) {
+                    if (pool_uu == project->pool_uuid) {
+                        pool_cache_monitor->update();
+                    }
+                },
+                *pool_cache_monitor));
+        pool_cache_monitor->signal_changed().connect([this] {
+            auto status = pool_cache_monitor->get_status();
+            pool_cache_window->refresh_list(status);
+            view_project.update_pool_cache_status(status);
+            part_browser_window->set_pool_cache_status(pool_cache_monitor->get_status());
+        });
+        pool_cache_monitor->update_now();
+        part_browser_window->signal_open_pool_cache_window().connect([this](auto &items) {
+            pool_cache_window->select_items(items);
+            pool_cache_window->present();
+        });
+
+
         project_needs_save = modified;
         set_view_mode(ViewMode::PROJECT);
+
+        if (view_project.update_meta() == false) {
+            Gtk::MessageDialog md(*this, "Metadata update required", false /* use_markup */, Gtk::MESSAGE_ERROR,
+                                  Gtk::BUTTONS_OK);
+            md.set_secondary_text(
+                    "Open and save the schematic for the project title to appear in the project manager.");
+            md.run();
+        }
     }
 }
 
@@ -1046,7 +1113,7 @@ void PoolProjectManagerAppWindow::handle_place_part(const UUID &uu)
     if (auto proc = find_top_schematic_process()) {
         auto pid = proc->proc->get_pid();
         allow_set_foreground_window(pid);
-        app->send_json(pid, {{"op", "place-part"}, {"part", (std::string)uu}});
+        app->send_json(pid, {{"op", "place-part"}, {"part", (std::string)uu}, {"time", gtk_get_current_event_time()}});
     }
 }
 
@@ -1055,7 +1122,7 @@ void PoolProjectManagerAppWindow::handle_assign_part(const UUID &uu)
     if (auto proc = find_top_schematic_process()) {
         auto pid = proc->proc->get_pid();
         allow_set_foreground_window(pid);
-        app->send_json(pid, {{"op", "assign-part"}, {"part", (std::string)uu}});
+        app->send_json(pid, {{"op", "assign-part"}, {"part", (std::string)uu}, {"time", gtk_get_current_event_time()}});
     }
 }
 
@@ -1133,7 +1200,7 @@ PoolProjectManagerProcess *PoolProjectManagerAppWindow::spawn(PoolProjectManager
         if (proc->proc) {
             auto pid = proc->proc->get_pid();
             Glib::RefPtr<PoolProjectManagerApplication>::cast_dynamic(get_application())
-                    ->send_json(pid, {{"op", "present"}});
+                    ->send_json(pid, {{"op", "present"}, {"time", gtk_get_current_event_time()}});
         }
         else {
             proc->win->present();
@@ -1305,7 +1372,7 @@ void PoolProjectManagerAppWindow::cleanup_pool_cache()
         md.run();
         return;
     }
-    std::set<std::pair<ObjectType, UUID>> items_needed;
+    ItemSet items_needed;
 
     PoolCached pool_cached(PoolManager::get().get_by_uuid(project->pool_uuid)->base_path,
                            project->pool_cache_directory);
@@ -1431,4 +1498,11 @@ void PoolProjectManagerAppWindow::cleanup_pool_cache()
     dia.run();
 }
 
+void PoolProjectManagerAppWindow::update_pool_cache_status_now()
+{
+    if (pool_cache_monitor) {
+        pool_cache_monitor->update_now();
+    }
+}
+
 } // namespace horizon
