@@ -7,6 +7,8 @@
 #include "pool_cache_window.hpp"
 #include "part_browser/part_browser_window.hpp"
 #include "widgets/pool_chooser.hpp"
+#include "widgets/project_meta_editor.hpp"
+#include "block/block.hpp"
 
 namespace horizon {
 
@@ -14,23 +16,25 @@ PoolProjectManagerViewCreateProject::PoolProjectManagerViewCreateProject(const G
                                                                          class PoolProjectManagerAppWindow *w)
     : win(w)
 {
-    builder->get_widget("create_project_name_entry", project_name_entry);
-    builder->get_widget("create_project_description_entry", project_description_entry);
     builder->get_widget("create_project_path_chooser", project_path_chooser);
     builder->get_widget("create_project_dir_label", project_dir_label);
     builder->get_widget("create_project_pool_combo", project_pool_combo);
 
-    project_name_entry->signal_changed().connect(sigc::mem_fun(*this, &PoolProjectManagerViewCreateProject::update));
-    project_description_entry->signal_changed().connect(
-            sigc::mem_fun(*this, &PoolProjectManagerViewCreateProject::update));
+    Gtk::Box *meta_box;
+    builder->get_widget("new_project_meta_box", meta_box);
+    meta_editor = Gtk::manage(new ProjectMetaEditor(meta_values));
+    meta_box->pack_start(*meta_editor, false, false, 0);
+    meta_editor->set_row_spacing(20);
+    meta_editor->show();
+    meta_editor->signal_changed().connect(sigc::mem_fun(*this, &PoolProjectManagerViewCreateProject::update));
     project_pool_combo->signal_changed().connect(sigc::mem_fun(*this, &PoolProjectManagerViewCreateProject::update));
     project_path_chooser->signal_file_set().connect(sigc::mem_fun(*this, &PoolProjectManagerViewCreateProject::update));
 }
 
 void PoolProjectManagerViewCreateProject::clear()
 {
-    project_name_entry->set_text("");
-    project_description_entry->set_text("");
+    meta_editor->clear();
+    meta_editor->preset();
     project_dir_label->set_text("");
     project_path_chooser->unselect_all();
 }
@@ -52,27 +56,24 @@ std::pair<bool, std::string> PoolProjectManagerViewCreateProject::create()
     std::string s;
     try {
         Project prj(UUID::random());
-        prj.name = project_name_entry->get_text();
-        prj.base_path = Glib::build_filename(project_path_chooser->get_file()->get_path(), prj.name);
-        prj.title = project_description_entry->get_text();
+        prj.base_path =
+                Glib::build_filename(project_path_chooser->get_file()->get_path(), meta_values.at("project_name"));
         prj.pool_uuid = static_cast<std::string>(project_pool_combo->get_active_id());
 
         auto app = Glib::RefPtr<PoolProjectManagerApplication>::cast_dynamic(win->get_application());
-        s = prj.create(PoolManager::get().get_by_uuid(prj.pool_uuid)->default_via);
+        s = prj.create(meta_values, PoolManager::get().get_by_uuid(prj.pool_uuid)->default_via);
         r = true;
     }
     catch (const std::exception &e) {
         r = false;
-        auto top = dynamic_cast<Gtk::Window *>(project_name_entry->get_ancestor(GTK_TYPE_WINDOW));
-        Gtk::MessageDialog md(*top, "Error creating project", false /* use_markup */, Gtk::MESSAGE_ERROR,
+        Gtk::MessageDialog md(*win, "Error creating project", false /* use_markup */, Gtk::MESSAGE_ERROR,
                               Gtk::BUTTONS_OK);
         md.set_secondary_text(e.what());
         md.run();
     }
     catch (const Glib::Error &e) {
         r = false;
-        auto top = dynamic_cast<Gtk::Window *>(project_name_entry->get_ancestor(GTK_TYPE_WINDOW));
-        Gtk::MessageDialog md(*top, "Error creating project", false /* use_markup */, Gtk::MESSAGE_ERROR,
+        Gtk::MessageDialog md(*win, "Error creating project", false /* use_markup */, Gtk::MESSAGE_ERROR,
                               Gtk::BUTTONS_OK);
         md.set_secondary_text(e.what());
         md.run();
@@ -83,9 +84,9 @@ std::pair<bool, std::string> PoolProjectManagerViewCreateProject::create()
 void PoolProjectManagerViewCreateProject::update()
 {
     bool valid = true;
-    if (project_name_entry->get_text().size() == 0)
+    if (meta_values.at("project_name").size() == 0)
         valid = false;
-    if (project_description_entry->get_text().size() == 0)
+    if (meta_values.at("project_title").size() == 0)
         valid = false;
     if (project_path_chooser->get_filenames().size() == 0)
         valid = false;
@@ -93,7 +94,7 @@ void PoolProjectManagerViewCreateProject::update()
         valid = false;
     if (valid) {
         auto base_path = project_path_chooser->get_file()->get_path();
-        auto dir = Glib::build_filename(base_path, project_name_entry->get_text());
+        auto dir = Glib::build_filename(base_path, meta_values.at("project_name"));
         project_dir_label->set_text(dir);
     }
     else {
@@ -110,12 +111,37 @@ PoolProjectManagerViewProject::PoolProjectManagerViewProject(const Glib::RefPtr<
     builder->get_widget("button_board", button_board);
     builder->get_widget("button_part_browser", button_part_browser);
     builder->get_widget("button_pool_cache", button_pool_cache);
-    builder->get_widget("entry_project_title", entry_project_title);
+    builder->get_widget("label_project_title", label_project_title);
+    builder->get_widget("label_project_author", label_project_author);
     builder->get_widget("label_pool_name", label_pool_name);
     builder->get_widget("label_pool_path", label_pool_path);
     builder->get_widget("label_project_directory", label_project_directory);
     builder->get_widget("prj_pool_change_button", button_change_pool);
     builder->get_widget("prj_pool_info_bar", pool_info_bar);
+    builder->get_widget("pool_cache_status_label", pool_cache_status_label);
+
+    Gtk::Button *open_button;
+    builder->get_widget("prj_open_dir_button", open_button);
+    open_button->signal_clicked().connect([this] {
+        auto uri = Gio::File::create_for_path(Glib::path_get_dirname(win->project_filename))->get_uri();
+        Gio::AppInfo::launch_default_for_uri(uri);
+    });
+
+    Gtk::Button *edit_button;
+    builder->get_widget("prj_edit_meta_button", edit_button);
+    edit_button->signal_clicked().connect([this] {
+        if (auto proc = win->find_top_schematic_process()) {
+            win->app->send_json(proc->proc->get_pid(), {{"op", "edit-meta"}});
+        }
+        else {
+            open_top_schematic();
+            if (auto proc2 = win->find_top_schematic_process()) {
+                proc2->signal_ready().connect([this, proc2] {
+                    win->app->send_json(proc2->proc->get_pid(), {{"op", "edit-meta"}});
+                });
+            }
+        }
+    });
 
     pool_info_bar->hide();
     auto reopen_button = pool_info_bar->add_button("Reopen project", 1);
@@ -128,8 +154,6 @@ PoolProjectManagerViewProject::PoolProjectManagerViewProject(const Glib::RefPtr<
         });
     });
 
-    entry_project_title->signal_changed().connect([this] { win->project_needs_save = true; });
-
     button_top_schematic->signal_clicked().connect(
             sigc::mem_fun(*this, &PoolProjectManagerViewProject::open_top_schematic));
     button_board->signal_clicked().connect(sigc::mem_fun(*this, &PoolProjectManagerViewProject::open_board));
@@ -179,8 +203,44 @@ void PoolProjectManagerViewProject::handle_button_part_browser()
 
 void PoolProjectManagerViewProject::handle_button_pool_cache()
 {
-    win->pool_cache_window->refresh_list();
+    win->update_pool_cache_status_now();
     win->pool_cache_window->present();
 }
 
+bool PoolProjectManagerViewProject::update_meta()
+{
+    auto top_block_filename = win->project->get_top_block().block_filename;
+    auto meta = Block::peek_project_meta(top_block_filename);
+    std::string title;
+    std::string author;
+    if (meta.count("project_title"))
+        title = meta.at("project_title");
+    if (meta.count("author"))
+        author = meta.at("author");
+    if (title.size())
+        win->set_title(title + " - Horizon EDA");
+    else
+        win->set_title("Project Manager");
+    label_project_title->set_text(title);
+    label_project_author->set_text(author);
+    return meta.size();
+}
+
+void PoolProjectManagerViewProject::update_pool_cache_status(const PoolCacheStatus &status)
+{
+    std::string txt;
+    if (status.n_current == status.n_total) {
+        txt = "All current";
+    }
+    else {
+        txt = std::to_string(status.n_out_of_date) + " out of date";
+    }
+    pool_cache_status_label->set_text(txt);
+}
+
+void PoolProjectManagerViewProject::reset_pool_cache_status()
+{
+    pool_cache_status_label->set_text("");
+}
+
 } // namespace horizon
