@@ -4,12 +4,13 @@
 #include <sigc++/sigc++.h>
 #include <string>
 #include "imp/action_catalog.hpp"
-#include "imp/action.hpp"
-#include "core/tool_id.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
 
+enum class ActionID;
+enum class ToolID;
+
 class CanvasPreferences {
 public:
     Appearance appearance;
@@ -51,6 +52,7 @@ class ZoomPreferences {
 public:
     bool smooth_zoom_2d = true;
     bool smooth_zoom_3d = false;
+    bool touchpad_pan = false;
 
     void load_from_json(const json &j);
     json serialize() const;
