@@ -62,7 +62,7 @@ Schematic::Schematic(const UUID &uu, const json &j, Block &iblock, Pool &pool)
     if (j.count("title_block_values")) {
         const json &o = j["title_block_values"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
-            title_block_values[it.key()] = it.value();
+            block->project_meta[it.key()] = it.value();
         }
     }
     if (j.count("pdf_export_settings")) {
@@ -80,7 +80,7 @@ Schematic::Schematic(const UUID &uu, const json &j, Block &iblock, Pool &pool)
 Schematic Schematic::new_from_file(const std::string &filename, Block &block, Pool &pool)
 {
     auto j = load_json_from_file(filename);
-    return Schematic(UUID(j["uuid"].get<std::string>()), j, block, pool);
+    return Schematic(UUID(j.at("uuid").get<std::string>()), j, block, pool);
 }
 
 Schematic::Schematic(const UUID &uu, Block &bl) : uuid(uu), block(&bl)
@@ -104,8 +104,10 @@ void Schematic::autoconnect_symbol(Sheet *sheet, SchematicSymbol *sym)
             if (!it_junc->second.bus && it_junc->second.position == pin_pos
                 && (it_junc->second.net || it_junc->second.connection_count > 0)) {
                 erase = true;
+                bool net_created = false;
                 if (it_junc->second.net == nullptr) {
                     it_junc->second.net = block->insert_net();
+                    net_created = true;
                 }
                 sym->component->connections.emplace(UUIDPath<2>(sym->gate->uuid, it_pin.first),
                                                     static_cast<Net *>(it_junc->second.net));
@@ -129,6 +131,9 @@ void Schematic::autoconnect_symbol(Sheet *sheet, SchematicSymbol *sym)
                     sheet->replace_junction(&it_junc->second, sym, &it_pin.second);
                 }
                 connected = true;
+
+                if (net_created)
+                    expand(true);
             }
             if (erase) {
                 sheet->junctions.erase(it_junc++);
@@ -416,7 +421,7 @@ bool Schematic::place_bipole_on_line(Sheet *sheet, SchematicSymbol *sym)
                 auto dst_swapped = (pin2_pos - from.get_position()).mag_sq() + (pin1_pos - to.get_position()).mag_sq();
                 SymbolPin *pin_from = &pin1;
                 SymbolPin *pin_to = &pin2;
-                if (dst_swapped > dst_normal) {
+                if (dst_swapped < dst_normal) {
                     std::swap(pin_from, pin_to);
                 }
                 auto connect_pin = [this, sheet, sym](SymbolPin &pin, const LineNet::Connection &conn, Net *net) {
@@ -531,7 +536,8 @@ void Schematic::expand(bool careful)
                                        [&sheet](const auto &a) { return sheet.texts.count(a.uuid) == 0; }),
                         texts.end());
         }
-        sheet.expand_symbols(*this);
+        if (!careful)
+            sheet.expand_symbols(*this);
     }
 
 
@@ -773,7 +779,7 @@ void Schematic::expand(bool careful)
     for (auto &it_sheet : sheets) {
         Sheet &sheet = it_sheet.second;
         if (sheet.pool_frame) {
-            std::map<std::string, std::string> values = title_block_values;
+            std::map<std::string, std::string> values = block->project_meta;
             for (const auto &it_v : sheet.title_block_values) {
                 values[it_v.first] = it_v.second;
             }
@@ -906,10 +912,38 @@ void Schematic::annotate()
     }
 }
 
+std::map<UUIDPath<2>, std::string> Schematic::get_unplaced_gates() const
+{
+    std::map<UUIDPath<2>, std::string> unplaced;
+    // collect gates
+    std::set<std::pair<const Component *, const Gate *>> gates;
+
+    // find all gates
+    for (const auto &it_component : block->components) {
+        for (const auto &it_gate : it_component.second.entity->gates) {
+            gates.emplace(&it_component.second, &it_gate.second);
+        }
+    }
+
+    // remove placed gates
+    for (auto &it_sheet : sheets) {
+        const auto &sheet = it_sheet.second;
+
+        for (const auto &it_sym : sheet.symbols) {
+            const auto &sym = it_sym.second;
+            gates.erase({sym.component, sym.gate});
+        }
+    }
+    for (const auto &it : gates) {
+        unplaced.emplace(std::piecewise_construct, std::forward_as_tuple(it.first->uuid, it.second->uuid),
+                         std::forward_as_tuple(it.first->refdes + it.second->suffix));
+    }
+    return unplaced;
+}
+
 Schematic::Schematic(const Schematic &sch)
     : uuid(sch.uuid), block(sch.block), name(sch.name), sheets(sch.sheets), rules(sch.rules),
-      title_block_values(sch.title_block_values), group_tag_visible(sch.group_tag_visible), annotation(sch.annotation),
-      pdf_export_settings(sch.pdf_export_settings)
+      group_tag_visible(sch.group_tag_visible), annotation(sch.annotation), pdf_export_settings(sch.pdf_export_settings)
 {
     update_refs();
 }
@@ -978,8 +1012,8 @@ json Schematic::serialize() const
     j["annotation"] = annotation.serialize();
     j["pdf_export_settings"] = pdf_export_settings.serialize_schematic();
     j["rules"] = rules.serialize();
-    j["title_block_values"] = title_block_values;
     j["group_tag_visible"] = group_tag_visible;
+    j["title_block_values"] = block->project_meta;
 
     j["sheets"] = json::object();
     for (const auto &it : sheets) {
