@@ -65,4 +65,10 @@ std::string format_m_of_n(unsigned int m, unsigned int n);
 std::string format_digits(unsigned int m, unsigned int digits_max);
 double parse_si(const std::string &inps);
 
+void rmdir_recursive(const std::string &dir_name);
+std::string replace_placeholders(const std::string &s, std::function<std::string(const std::string &)> fn,
+                                 bool keep_empty);
+
+Coordi dir_from_arrow_key(unsigned int key);
+
 } // namespace horizon
